package com.noorija.secret.guard

import android.app.Activity
import android.app.Application
import android.os.Bundle
import com.noorija.secret.guard.text.Command
import com.noorija.secret.guard.util.PreferencesUtil
import org.webrtc.PeerConnectionFactory
import timber.log.Timber
import java.util.Date

enum class AppStatus {
    BACKGROUND, RETURN_TO_FOREGROUND, FOREGROUND
}

class SecretGuardApp : Application() {
    companion object {
        var instance: SecretGuardApp? = null
        const val FORE_CHANNEL_ID = "forenotification"
    }

    private var _appStatus: AppStatus = AppStatus.FOREGROUND
    var appStatus : AppStatus
        get() = _appStatus
        set(value) {
            _appStatus = value
        }

    private var _fcmRegId: String = ""
    var fcmRegId : String
        get() = _fcmRegId
        set(value) {
            _fcmRegId = value
        }

    override fun onCreate() {
        super.onCreate()

        instance = this
        PreferencesUtil.init(applicationContext)
        firebaseToken()
        registerActivityLifecycleCallbacks(NSGActivityLifecycleCallbacks());

        if (BuildConfig.DEBUG) {
            Timber.plant(Timber.DebugTree())
        }
    }

    override fun onTerminate() {
        super.onTerminate()
        instance = null
    }

    private fun firebaseToken() {
//        FirebaseMessaging.getInstance().token.addOnCompleteListener { task ->
//            if(task.isSuccessful) {
//                _fcmRegId = task.result?:""
//            }
//        }
    }

    private var END_WAIT_MINUTE: Long = 60 * 60 * 1000
    fun isAppRestart() : Boolean {
        if(appStatus == AppStatus.RETURN_TO_FOREGROUND) {
            val saveDate = PreferencesUtil.getLong(Command.SECRET_GUARD_PREF_BACKGROUND_PAUSE_TIME, 0L)
            val nowDate = Date().getTime()
            val compareDate = (nowDate - saveDate) / END_WAIT_MINUTE
            if (saveDate != 0L && compareDate >= 1) {
                return true
            }
        }
        return false
    }

    inner class NSGActivityLifecycleCallbacks : ActivityLifecycleCallbacks {
        private var running = 0
        override fun onActivityCreated(activity: Activity, savedInstanceState: Bundle?) {}
        override fun onActivityStarted(activity: Activity) {
            if (++running == 1) {
                appStatus = AppStatus.RETURN_TO_FOREGROUND
            } else if (running > 1) {
                appStatus = AppStatus.FOREGROUND
            }
        }

        override fun onActivityResumed(activity: Activity) {}
        override fun onActivityPaused(activity: Activity) {}
        override fun onActivityStopped(activity: Activity) {
            if (--running == 0) {
                appStatus = AppStatus.BACKGROUND
                val nowDate = Date().time
                PreferencesUtil.putPref(Command.SECRET_GUARD_PREF_BACKGROUND_PAUSE_TIME, nowDate)
            }
        }

        override fun onActivitySaveInstanceState(activity: Activity, outState: Bundle) {}
        override fun onActivityDestroyed(activity: Activity) {}
    }

}