package com.noorija.secret.guard.manager

import android.content.Context
import android.os.Handler
import androidx.databinding.ObservableArrayList
import io.reactivex.Completable
import io.reactivex.CompletableTransformer
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.CompositeDisposable
import io.reactivex.disposables.Disposable
import io.reactivex.schedulers.Schedulers
import timber.log.Timber
import ua.naiksoftware.stomp.Stomp
import ua.naiksoftware.stomp.StompClient
import ua.naiksoftware.stomp.dto.LifecycleEvent
import ua.naiksoftware.stomp.dto.StompHeader
import java.util.*

class FindCallSocketHandler (context: Context)/* : Thread()*/ {
    private val topicList = ObservableArrayList<Disposable>()

    private var stompClient = Stomp.over(Stomp.ConnectionProvider.OKHTTP, FindCallStompManger.RTC_HOST_URL)
//    private val stompClient : StompClient by lazy {
//        Stomp.over(Stomp.ConnectionProvider.OKHTTP, FindCallStompManger.RTC_HOST_URL)
//    }
//    private lateinit var stompClient : StompClient
    private var mRestPingDisposable: Disposable = CompositeDisposable()
    private var compositeDisposable: CompositeDisposable = CompositeDisposable()

    private val headers: MutableList<StompHeader> = ArrayList()

    private var reConnCnt = 0 // 5초 간격 재 접속 시도(10회)
    private val maxConnCnt = 10
    private val delayTime = 5000L

    fun init() {
    }

    fun getStompClient(): StompClient {
        return stompClient
    }

    fun connectStomp() {
        stompClient.withClientHeartbeat(0).withServerHeartbeat(1000)
        resetSubscriptions()

        val dispLifecycle = stompClient.lifecycle()
            .subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread())
            .subscribe { lifecycleEvent: LifecycleEvent ->
                when (lifecycleEvent.type) {
                    LifecycleEvent.Type.OPENED -> {
                        Timber.d("Stomp connection opened")
                        reConnCnt = 0
                        restartTopic()
//                        EventBus.getInstance()?.post(SocketModel(SocketEventType.SOCKET_CONNECTED, 0))
                    }
                    LifecycleEvent.Type.ERROR -> {
                        Timber.e("Stomp connection error\n${lifecycleEvent.exception.toString()}")
                        //reconnectStomp()
                    }
                    LifecycleEvent.Type.CLOSED -> {
                        Timber.d("Stomp connection closed")
                        reconnectStomp()
//                        EventBus.getInstance()?.post(SocketModel(SocketEventType.SOCKET_CLOSED, 8))
                    }
                    LifecycleEvent.Type.FAILED_SERVER_HEARTBEAT -> {
                        Timber.d("Stomp failed server heartbeat")
                        reconnectStomp()
//                        EventBus.getInstance()?.post(SocketModel(SocketEventType.SOCKET_FAILED_SERVER_HEARTBEAT, 7))
                    }
                }
            }

        compositeDisposable.add(dispLifecycle)

        stompClient.connect(headers)
    }

    fun disconnectStomp() {
        mRestPingDisposable.dispose()
        compositeDisposable.dispose()
        stompClient.disconnect()
        Timber.d("Stomp disconnection")
    }

    private fun reconnectStomp() {
        reConnCnt += 1
        Timber.d("Stomp reconnectStomp reConnCnt >>>>>>>>>> $reConnCnt")
        if(reConnCnt > maxConnCnt) {
            disconnectStomp()
            return
        }
        Handler().postDelayed({
            stompClient.connect(headers)
        }, delayTime)
    }

    fun isConnected() : Boolean {
        return stompClient.isConnected
    }

    fun addTopic(dispTopic: Disposable) {
        topicList.add(dispTopic)
        compositeDisposable.add(dispTopic)
    }

    private fun restartTopic() {
        for(dispTopic in topicList) {
            compositeDisposable.add(dispTopic)
        }
    }

    fun clearTopic() {
        compositeDisposable.clear()
    }

    fun addListener(listener: StompClient.OnStompClientListener) {
        stompClient.addOnStompClientListener(listener)
    }

    private fun resetSubscriptions() {
        compositeDisposable.dispose()
        compositeDisposable = CompositeDisposable()
    }

    private fun applySchedulers(): CompletableTransformer {
        return CompletableTransformer { upstream: Completable ->
            upstream
                .unsubscribeOn(Schedulers.newThread())
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
        }
    }

}