package com.noorija.secret.guard.manager

import android.annotation.SuppressLint
import android.app.PendingIntent
import android.content.Context
import android.content.Intent
import com.google.gson.Gson
import com.google.gson.GsonBuilder
import com.google.gson.reflect.TypeToken
import com.noorija.secret.guard.model.StompPayload
import com.noorija.secret.guard.net.FindCallSocketManager
import com.noorija.secret.guard.net.OnStompMessageListener
import com.noorija.secret.guard.rest.RoomIdInfo
import com.noorija.secret.guard.util.StringUtil
import com.noorija.secret.guard.view.RcvMsgActivity
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import timber.log.Timber
import ua.naiksoftware.stomp.StompClient
import ua.naiksoftware.stomp.dto.LifecycleEvent
import ua.naiksoftware.stomp.dto.StompMessage
import java.lang.reflect.Type


open class FindCallStompManger private constructor(mContext: Context) {
    companion object {
        fun createFindCallStompManger(context: Context): FindCallStompManger {
            return FindCallStompManger(context)
        }
        const val RTC_HOST_URL = "wss://kosk.co.kr/signaling/websocket"
        const val RTC_SUBSCRIBE_URI_MSG_SEND = "/topic/msgSend"
        const val RTC_SUBSCRIBE_URI_CANDIDATE = "/topic/peer/iceCandidate"
        const val RTC_SUBSCRIBE_URI_OFFER = "/topic/peer/offer"
        const val RTC_SUBSCRIBE_URI_ANSWER = "/topic/peer/answer"
        const val RTC_SUBSCRIBE_URI_CALL_KEY = "/topic/call/key"
        const val RTC_SUBSCRIBE_URI_SEND_KEY = "/topic/send/key"
        const val RTC_APP_URI_SEND_KEY = "/app/send/key"
    }

    private val context = mContext

    /*private val stompManager by lazy {
        FindCallStompManger.createFindCallStompManger(context)
    }*/
    private lateinit var stompHandler: FindCallSocketManager
    private lateinit var stompClient: StompClient

    private lateinit var rooms: List<RoomIdInfo>

    private lateinit var stompMessageListener: OnStompMessageListener

    fun init(strRooms : String) {
        stompHandler = FindCallSocketManager(context)
        stompClient = stompHandler.getStompClient()
        createSocket(strRooms)
    }

    private fun createSocket(strRooms: String) {
        stompHandler.run()
        val listType: Type = object : TypeToken<ArrayList<RoomIdInfo>>() {}.type
        rooms = Gson().fromJson(strRooms, listType)
        for(item: RoomIdInfo in rooms) {
            val myKey = StringUtil.getRandomString(36).substring(2, 14)
            item.myKey = myKey
//            addDispTopic("$RTC_SUBSCRIBE_URI_CANDIDATE/$myKey/${item.roomId}")
//            addDispTopic("$RTC_SUBSCRIBE_URI_OFFER/$myKey/${item.roomId}")
            addDispTopic("$RTC_SUBSCRIBE_URI_MSG_SEND/${item.roomId}")
        }
    }

    private fun addDispTopic(destinationPath: String) {
        if (destinationPath.isNotBlank()) {
            val dispTopic = stompClient.topic(destinationPath)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe({
                    onStompDataSuccess(it)
                }) { throwable: Throwable ->
                    onStompDataError(throwable)
                    Timber.e("Error on subscribe topic\n${throwable}}")
                    Timber.e("Error on subscribe topic\n${throwable.printStackTrace()}}")
                    Timber.e("Error on subscribe topic\n${throwable.message}}")
                }
            stompHandler.addTopic(dispTopic)
        }
    }

    private fun onStompDataSuccess(data: StompMessage) {
        val gson = GsonBuilder()
            .disableHtmlEscaping()
            .serializeNulls()
            .create()
        val message = gson.fromJson(data.payload, StompPayload::class.java)
        Timber.d("StompDataSuccess >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> $message")
        for(item: RoomIdInfo in rooms) {
            if(item.myKey != message.mykey) {
                message.mykey = item.myKey
                stompMessageListener.onMessageReceived(item.nickNm, message)
            }
        }
    }

    @SuppressLint("CheckResult")
    fun sendStompMessage(message: String, strPayload: String) {
        Timber.d("FindCallStompManger >>>>>>>>>>>>>>>>>>>>> $message >>> $strPayload")
        val gson = GsonBuilder()
            .disableHtmlEscaping()
            .serializeNulls()
            .create()
        val payload = gson.fromJson(strPayload, StompPayload::class.java)
        payload.msgDesc = message
        payload.currentDate = StringUtil.getCurrentDate()
        Timber.d("sendStompMessage >>>>>>>>>>>>>>>>>>>>> $payload")
        stompClient.send("$RTC_SUBSCRIBE_URI_MSG_SEND/${payload.roomId}", gson.toJson(payload, StompPayload::class.java)).subscribe { }
    }

    private fun onRcvMsg(message : StompPayload) {
        Timber.d("onRcvMsg >>> $message")
        val intent = Intent(context, RcvMsgActivity::class.java)
        val pi = PendingIntent.getActivity(context, 0, intent, PendingIntent.FLAG_UPDATE_CURRENT)
        pi.send()
    }

    private fun onStompDataError(t: Throwable) {
        Timber.d("${t.printStackTrace()}")
    }

    fun addListener(listener: OnStompMessageListener) {
        this.stompMessageListener = listener
    }

}