package com.noorija.secret.guard.rest

import com.google.gson.Gson
import com.google.gson.GsonBuilder
import okhttp3.Interceptor
import okhttp3.OkHttpClient
import okhttp3.Request
import okhttp3.Response
import okhttp3.logging.HttpLoggingInterceptor
import retrofit2.Retrofit
import retrofit2.adapter.rxjava2.RxJava2CallAdapterFactory
import retrofit2.converter.gson.GsonConverterFactory
import retrofit2.converter.scalars.ScalarsConverterFactory
import java.io.IOException
import java.util.concurrent.TimeUnit

object RetrofitClient {
    private const val ALL_TIMEOUT = 60L
//    private val REST_SERVER_URL = SecretGuardApp.instance!!.getString(R.string.rest_server_url)
    private const val REST_SERVER_URL = "https://kosk.co.kr"

    private val gson: Gson = GsonBuilder()
        .setLenient()
        .create()

    private val client: OkHttpClient = OkHttpClient.Builder()
        .connectTimeout(ALL_TIMEOUT, TimeUnit.SECONDS)
        .writeTimeout(ALL_TIMEOUT, TimeUnit.SECONDS)
        .readTimeout(ALL_TIMEOUT, TimeUnit.SECONDS)
        .addInterceptor(setHttpLoggingInterceptor())
//        .addInterceptor(EncodeInterceptor())
        .build()

    private val retrofit: Retrofit = Retrofit.Builder()
        .baseUrl(REST_SERVER_URL)
        .client(client)
        .addCallAdapterFactory(RxJava2CallAdapterFactory.create())
        .addConverterFactory(ScalarsConverterFactory.create())
        .addConverterFactory(GsonConverterFactory.create(gson))
        .build()

    private fun setHttpLoggingInterceptor(): HttpLoggingInterceptor {
        return HttpLoggingInterceptor().setLevel(HttpLoggingInterceptor.Level.BODY)
    }

    internal class EncodeInterceptor : Interceptor {
        @Throws(IOException::class)
        override fun intercept(chain: Interceptor.Chain): Response {
            val request: Request = chain.request()
            var stringurl = request.url.toString()
            stringurl = stringurl.replace("%3D", "=")
            val newRequest: Request = Request.Builder()
                .url(stringurl)
                .build()
            return chain.proceed(newRequest)
        }
    }

    fun createInterface(): RetrofitInterface = retrofit.create(RetrofitInterface::class.java)
}