package com.noorija.secret.guard.rest

import com.noorija.secret.guard.text.Command
import io.reactivex.Single
import retrofit2.http.Body
import retrofit2.http.GET
import retrofit2.http.Header
import retrofit2.http.POST
import retrofit2.http.Query

interface RetrofitInterface {
    /*AppMenuController Controller*/
    // 메뉴 리스트
    @GET("/api/menu/menuList")
    fun menuList(
        @Header(Command.PARAMETER_X_AUTH_HEADER) appKey: String,
        @Query("langCd") langCd : String
    ): Single<AppMenuListResponse>

    /*InformationController Controller*/
    // FAQ 리스트List
    @GET("/api/info/faqList")
    fun faqList(
        @Header(Command.PARAMETER_X_AUTH_HEADER) appKey: String,
        @Query("langCd") langCd : String
    ): Single<FaqListResponse>

    // 문의메일 등록
    @POST("/api/info/sendAsk")
    fun sendAsk(
        @Header(Command.PARAMETER_X_AUTH_HEADER) appKey: String,
        @Body reqData : AskRequest
    ): Single<SaveResponse>

    // 공지사항 리스트List
    @POST("/api/info/annList")
    fun annList(
        @Header(Command.PARAMETER_X_AUTH_HEADER) appKey: String,
        @Body reqData : AnnounceListRequest
    ): Single<PublicListResponse>

    /*MembSettingController Controller*/
    // APP 환경 세팅정보
    @GET("/api/sett/info-setting")
    fun infoSetting(
        @Header(Command.PARAMETER_X_AUTH_HEADER) appKey: String,
        @Query("langCd") langCd : String
    ): Single<AppMenuListResponse>

    // APP 환경 세팅정보 저장
    @POST("/api/sett/save-setting")
    fun saveSetting(
        @Header(Command.PARAMETER_X_AUTH_HEADER) appKey: String,
        @Body reqData : MembSettingRequest
    ): Single<MembSettingResponse>

    /*QrMngController Controller*/
    // QR 등록 가능 정보
    @POST("/api/qrInfo/regUseQr")
    fun regUseQr(
        @Header(Command.PARAMETER_X_AUTH_HEADER) appKey: String,
        @Body reqData : QrUseRegRequest
//        @Query("qrNo") qrNo : String,
//        @Query("langCd") langCd : String
    ): Single<QRExistResponse>

    // PUB QR List
    @GET("/api/qrInfo/pubQrList")
    fun pubQrList(
        @Header(Command.PARAMETER_X_AUTH_HEADER) appKey: String,
        @Query("langCd") langCd : String
    ): Single<PublicQrListResponse>

    // My QR List
    @GET("/api/qrInfo/myQrList")
    fun myQrList(
        @Header(Command.PARAMETER_X_AUTH_HEADER) appKey: String,
        @Query("langCd") langCd : String
    ): Single<MyQrListResponse>

    // 등록QR 차량정보 저장
    @POST("/api/qrInfo/save-qrinfo")
    fun saveQrinfo(
        @Header(Command.PARAMETER_X_AUTH_HEADER) appKey: String,
        @Body reqData : QrMembRegRequest
    ): Single<SaveResponse>

    // QR 등록
    @POST("/api/qrInfo/regQr")
    fun regQr(
        @Header(Command.PARAMETER_X_AUTH_HEADER) appKey: String,
        @Body reqData : QrMembRegRequest
    ): Single<QrRegResponse>

    // 등록QR 차량정보 삭제
    @POST("/api/qrInfo/delete-qrinfo")
    fun deleteQrinfo(
        @Header(Command.PARAMETER_X_AUTH_HEADER) appKey: String,
        @Body reqData : QrMembDelRequest
    ): Single<SaveResponse>

    // 호출 List
    @POST("/api/qrInfo/call-hist")
    fun callHist(
        @Header(Command.PARAMETER_X_AUTH_HEADER) appKey: String,
        @Body reqData : CallHistoryRequest
    ): Single<CallHistoryResponse>

    /*SessionController Controller*/
    // QR생성.
    @GET("/api/session/makeQR")
    fun makeQR(
        @Header(Command.PARAMETER_X_AUTH_HEADER) appKey: String,
        @Query("langCd") langCd : String,
        @Query("currentDate") currentDate : String
    ): Single<QRImgUrlResponse>

    // 시스템 튜토리얼 리스트
    @GET("/api/session/getTutorial")
    fun getTutorial(
        @Query("langCd") langCd : String
    ): Single<TutoListResponse>

    // 약관 리스트 casTyCd A 최초 설치시
    @GET("/api/session/getCas")
    fun getCas(
        @Query("langCd") langCd : String,
        @Query("casTyCd") casTyCd : String = "A"
    ): Single<CasMngListResponse>

    // 약관 리스트 casTyCd A 최초 설치시
    @GET("/api/session/getAppkey")
    fun getAppkey(
        @Query("phoneNo") phoneNo : String,
        @Query("deviId") deviId : String,
        @Query("langCd") langCd : String
    ): Single<JwtResponse>

    // 시스템 최종 버전 정보
    @POST("/api/session/getVersion")
    fun getVersion(
        @Body reqData : VerInfoRequest
    ): Single<SysVersionResponse>

    // 가입 처리
    @POST("/api/session/createSg")
    fun createSg(
        @Body reqData : MembRequest
    ): Single<JwtResponse>

    /*WaitWebController Controller*/
    // QR 호출하기 Web 최초화면 정보
    @GET("/api/web/wait-int")
    fun getWaitInt(
        @Header(Command.PARAMETER_X_AUTH_HEADER) appKey: String,
        @Query("qrNo ") qrNo : String,
        @Query("langCd") langCd : String
    ): Single<WaitCallInfoResponse>

    // 호출 문자 정보
    @GET("/api/web/call-req-msg")
    fun callReqMsg(
        @Header(Command.PARAMETER_X_AUTH_HEADER) appKey: String,
        @Query("qrNo ") qrNo : String,
        @Query("langCd") langCd : String
    ): Single<CallReqListResponse>

    // 보이스톡/문자 호출 답변 전송
    @GET("/api/web/call-reply-send")
    fun callReplySend(
        @Header(Command.PARAMETER_X_AUTH_HEADER) appKey: String,
        @Query("callReplySendRequest ") callReplySendRequest : CallReqSendRequest
    ): Single<SaveResponse>

    // 호출 답변 문자 정보
    @GET("/api/web/call-reply-msg")
    fun callReplyMsg(
        @Header(Command.PARAMETER_X_AUTH_HEADER) appKey: String,
        @Query("qrNo ") qrNo : String
    ): Single<CallReqListResponse>

    // 보이스톡 호출자가 취소
    @GET("/api/web/call-cancel-send")
    fun callCancelSend(
        @Header(Command.PARAMETER_X_AUTH_HEADER) appKey: String,
        @Query("callCancelSendRequest ") callCancelSendRequest : CallCancelSendRequest
    ): Single<SaveResponse>

    // 호출 보이스톡/문자 전송
    @POST("/api/web/call-req-send")
    fun callReqSend(
        @Header(Command.PARAMETER_X_AUTH_HEADER) appKey: String,
        @Body reqData : CallReqSendRequest
    ): Single<CallResponse>

}