package com.noorija.secret.guard.rest

import android.os.Parcelable
import androidx.databinding.ObservableBoolean
import kotlinx.parcelize.Parcelize

data class FieldError(val field: String, val value: String, val reason: String)
data class ExceptionDTO(val status: Int, val code: String, val message: String, val path: String, val errors: List<FieldError>)

data class SaveResponse(val code: Int, val message: String)

data class AppMenuInfo(val appMenuNo: String, val appMenuNm: String, val useYn: String, val viewNum: Int, val filePath: String)
data class RoomIdInfo(val roomId: String, val nickNm: String, var myKey: String)
data class AppMenuListResponse(val code: String, val message: String, val listMenu: List<AppMenuInfo>, val mainMenu: List<AppMenuInfo>, val roomIdList: List<RoomIdInfo>)

data class SysVersionResponse(val code: String, val message: String, val upRequireInfo: String, val langCd: String, val tutoStart: String, val appKey: String)

data class JwtResponse(val code: String, val message: String, val appKey: String, val langCd: String, val membNo: String)

data class MembSettingResponse(var code: String, var message: String, var alarmUseYn: String, var alarmIntervalCd: String, var alarmIntervalNm: String
                                , var alarmSoundYn: String, var alarmSoundNm: String, var alarmVibYn: String, var callPhoneYn: String
                                , var callMsgYn: String, var allowYn: String)

data class QRImgUrlResponse(val code: String, val message: String, val qrImgUrl: String)

data class TutoInfo(val filePath: String, val viewNum: Int)
data class TutoListResponse(val code: String, val message: String, val tutoDataList: List<TutoInfo>)

data class CasMngInfo(val casNo: String, val casNm: String, val casDesc: String, val requiredYn: String, val viewNum: Int
                      , var checked: ObservableBoolean, var isAgree: Boolean = false)
data class CasMngListResponse(val code: String, val message: String, val casMngDataList: List<CasMngInfo>)

data class QrRegResponse(val code: String, val message: String, val subscr: String)

data class QRExistResponse(val code: String, val message: String, val regQr: String) /*등록여부 - N 신규등록, D 등록신청*/

@Parcelize
data class MyQrInfo(val membNo: String, val qrOwnerCd: String, val qrOwnerNm: String, val qrUseCd: String, val qrUseNm: String
                    , val makerCar: String, val carInfo: String, val qrNo: String, val addDt: String, val nickNm: String, val filePath: String) : Parcelable
data class MyQrListResponse(val code: String, val message: String, val adOutYn: String, val myQrList: List<MyQrInfo>)

data class SubQrInfo(val membNo: String, val qrOwnerCd: String, val qrOwnerNm: String, val nickNm: String, val addDt: String, val delUseYn: String)
data class MyPublicQrInfo(val membNo: String, val qrOwnerCd: String, val qrOwnerNm: String, val carInfo: String
                    , val qrNo: String, val nickNm: String, val filePath: String, val delUseYn: String, val subList: List<SubQrInfo>)
data class PublicQrListResponse(val code: String, val message: String, val adOutYn: String, val publicQrList: List<MyPublicQrInfo>)

data class CallHistory(val membNo: String, val nickNm: String, val callTypeCd: String, val callTypeNm: String, val callSeq: String
                       , val callReqMsg: String, val callStatCd: String, val callStatNm: String, val callStatDt: String)
data class CallHistoryResponse(val code: String, val message: String, val adOutYn: String, val totalSize: Int, val  callHistList: List<CallHistory>)

data class WaitCallInfoResponse(val code: String, val message: String, val callPhoneYn: String, val callMsgYn: String, val waitCallMsg: String)

data class ReqInfo(val codeDtlCd: String, val codeDtlNm: String, val viewNum: String)
data class CallReqListResponse(val code: String, val message: String, val custSubscr: String, val callSeq: String, val casMngDataList: List<ReqInfo>)

data class ReplyInfo(val codeDtlCd: String, val codeDtlNm: String, val viewNum: String)
data class ReplyListResponse(val code: String, val message: String, val casMngDataList: List<ReplyInfo>)

data class CallResponse(val code: String, val message: String, val custSubscr: String, val custCallRoom: String, val callSeq: String)

data class FaqInfo(val faqNo: String, val faqTypeNm: String, val faqSubject: String, val faqDesc: String, var isExpanded: Boolean = false)
data class FaqListResponse(val code: String, val message: String, val faqInfoList: List<FaqInfo>)

data class PublicInfo(val annNo: String, val annNm: String, val annDesc: String)
data class PublicListResponse(val code: String, val message: String, val totalSize: Int, val publicList: List<PublicInfo>)
