package com.noorija.secret.guard.service

import android.app.NotificationChannel
import android.app.NotificationManager
import android.app.NotificationManager.IMPORTANCE_LOW
import android.app.PendingIntent
import android.content.Context
import android.content.Intent
import android.os.Binder
import android.os.Build
import android.os.IBinder
import androidx.annotation.RequiresApi
import androidx.core.app.NotificationCompat
import androidx.lifecycle.LifecycleService
import com.noorija.secret.guard.SecretGuardApp
import com.noorija.secret.guard.manager.FindCallRtcManager
import com.noorija.secret.guard.view.MainActivity

class FindCallServiceBackup: LifecycleService() {

    // 서비스 바인딩을 위한 Binder 클래스
    private val localBinder by lazy { LocalBinder() }

    private val fndCallRtcManager by lazy {
        FindCallRtcManager.createFindCallRtcManager(this)
    }

    val intent = Intent(this, MainActivity::class.java)
    private val pendingIntent: PendingIntent? = PendingIntent.getActivity(
        this, //context
        0, //request code
        intent, //flag
        PendingIntent.FLAG_IMMUTABLE //flag
    )

    private val notificationBuilder by lazy {
        NotificationCompat.Builder(this, SecretGuardApp.FORE_CHANNEL_ID)
            .setContentTitle("foreground")
            .setContentText("진행 상황 : 0")
            .setContentIntent(pendingIntent)
    }

//    private val notificationManager by lazy {
//        getSystemService(NotificationManager::class.java)
////        getSystemService(Context.NOTIFICATION_SERVICE) as NotificationManager
//    }

    private lateinit var notificationManager: NotificationManager

    private var isSvcRun = true

    override fun onCreate() {
        super.onCreate()
        // 서비스 처음 생성 시 수행할 작업
        initValues()
    }

    override fun onStartCommand(intent: Intent?, flags: Int, startId: Int): Int {
        super.onStartCommand(intent, flags, startId)
        intent?.let {
            when (it.action) {
                ACTION_START_SERVICE -> {
                    startForegroundService()
                }
                ACTION_STOP_SERVICE -> {
                    stopService()
                }
            }
        }
        return START_NOT_STICKY
    }

    override fun onBind(intent: Intent): IBinder? {
        super.onBind(intent)
        isSvcRun = true
        FindCallBindService.bindService()
        return localBinder
//        return null
    }

    private fun handleBind() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            // API 26 이상은 포그라운드 서비스 시작
            startForegroundService(Intent(this@FindCallServiceBackup, FindCallServiceBackup::class.java))
        } else {
            // API 26 미만은 일반 서비스로 시작
            startService(Intent(this@FindCallServiceBackup, FindCallServiceBackup::class.java))
        }
        startForeground(NOTIFICATION_ID, notificationBuilder.build())
    }

    override fun onUnbind(intent: Intent?): Boolean {
        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.O) {
            stopSelf()
        }
        stopForeground(true)
        isSvcRun = false
        return true
    }

    override fun onDestroy() {
        super.onDestroy()
        if(isSvcRun) {
            stopService(Intent(this, FindCallServiceBackup::class.java))
            stopForeground(true)
            isSvcRun = false
        }
    }

    private fun initValues() {
        fndCallRtcManager.initModule()
    }

    private fun startForegroundService() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            createNotificationChannel()
        }
        startForeground(NOTIFICATION_ID, notificationBuilder.build())
    }

    private fun stopService() {
        isSvcRun = false
        initValues()
        (getSystemService(Context.NOTIFICATION_SERVICE) as NotificationManager).cancel(
            NOTIFICATION_ID
        )
        stopForeground(true)
        stopSelf()
    }

    @RequiresApi(Build.VERSION_CODES.O)
    private fun createNotificationChannel() {
        val channel =
            NotificationChannel(
                NOTIFICATION_CHANNEL_ID,
                NOTIFICATION_CHANNEL_NAME,
                NotificationManager.IMPORTANCE_DEFAULT
            )

        notificationManager = getSystemService(NotificationManager::class.java)

        notificationManager.createNotificationChannel(channel)
    }

    // IBinder를 반환하는 Binder 클래스
    // 클라이언트와 서비스 사이의 통신을 위한 Binder 클래스입니다.
    // 클라이언트는 이 Binder를 사용하여 서비스의 메서드를 호출할 수 있습니다.
    // 왜 internal inner class로 선언했는지 궁금하시다면, 이는 Binder 클래스가 Binder()를 상속받기 때문입니다.
    // Binder 클래스는 자바에서만 사용할 수 있기 때문에, Kotlin에서는 Binder()를 상속받아 사용할 수 없습니다.
    // 따라서 Binder 클래스를 내부 클래스로 선언하여 사용하였습니다.
    // 참고로, Binder 클래스는 android.os 패키지에 존재합니다.
    internal inner class LocalBinder : Binder() {
        fun getService(): FindCallServiceBackup = this@FindCallServiceBackup
    }

}