package com.noorija.secret.guard.util

import android.content.Context
import android.content.SharedPreferences
import com.noorija.secret.guard.text.Command

object PreferencesUtil {

    private lateinit var prefs: SharedPreferences

    fun init(context: Context) {
        prefs = context.getSharedPreferences(Command.SECRET_GUARD_PREF_NAME, Context.MODE_PRIVATE)
    }

    private inline fun SharedPreferences.edit(operation: (SharedPreferences.Editor) -> Unit) {
        val editor = edit()
        operation(editor)
        editor.apply()
    }

    fun getString(key: String, defValue: String = ""): String {
        return prefs.getString(key, defValue)!!
    }

    fun getInt(key: String, defValue: Int = -1): Int {
        return prefs.getInt(key, defValue)
    }

    fun getBoolean(key: String, defValue: Boolean = false): Boolean {
        return prefs.getBoolean(key, defValue)
    }

    fun getLong(key: String, defValue: Long = -1L): Long {
        return prefs.getLong(key, defValue)
    }

    fun getFloat(key: String, defValue: Float = -1F): Float {
        return prefs.getFloat(key, defValue)
    }

    fun getPref(key: String, defValue: Any) {
        when(defValue) {
            is String   -> prefs.getString  (key, defValue)
            is Int      -> prefs.getInt     (key, defValue)
            is Boolean  -> prefs.getBoolean (key, defValue)
            is Long     -> prefs.getLong    (key, defValue)
            is Float    -> prefs.getFloat   (key, defValue)
        }
    }

    fun putPref(key: String, value: Any) {
        val it = prefs.edit()
        when(value) {
            is String   -> it.putString     (key, value).apply()
            is Int      -> it.putInt        (key, value).apply()
            is Boolean  -> it.putBoolean    (key, value).apply()
            is Long     -> it.putLong       (key, value).apply()
            is Float    -> it.putFloat      (key, value).apply()
        }
    }

    fun remove(key: String = "") {
        val it = prefs.edit()
        if(StringUtil.isBlank(key)) it.clear() else it.remove(key)
        it.commit()
    }

}