package com.noorija.secret.guard.util

import android.Manifest
import android.annotation.SuppressLint
import android.app.Activity
import android.content.Context
import android.content.pm.PackageInfo
import android.content.pm.PackageManager
import android.os.Build
import android.telephony.TelephonyManager
import android.util.Base64
import android.widget.Toast
import androidx.core.app.ActivityCompat
import com.google.gson.GsonBuilder
import timber.log.Timber
import java.security.MessageDigest
import java.security.NoSuchAlgorithmException
import com.noorija.secret.guard.R
import java.text.SimpleDateFormat
import java.time.LocalDate
import java.time.LocalDateTime
import java.time.format.DateTimeFormatter
import java.util.Date

object StringUtil {
    fun isBlank(str: String): Boolean {
        return str == null || str.equals("")
    }

    @SuppressLint("HardwareIds")
    fun getPhoneNumber(context: Context, isHyphen: Boolean = false) : String {
        var telNumber = ""
        val mTelephonyManager = context.getSystemService(Context.TELEPHONY_SERVICE) as TelephonyManager
        if(mTelephonyManager != null) {
            if(ActivityCompat.checkSelfPermission(context, Manifest.permission.READ_PHONE_STATE) != PackageManager.PERMISSION_GRANTED) {
                showToast(context, context.getString(R.string.msg_permission_denied_phone))
            } else if(mTelephonyManager.getSimState() == TelephonyManager.SIM_STATE_UNKNOWN
                || mTelephonyManager.getSimState() == TelephonyManager.SIM_STATE_ABSENT){
                showToast(context, context.getString(R.string.toast_msg_empty_usim))
            } else {
                telNumber = mTelephonyManager.line1Number
                telNumber = telNumber.replace("+82", "0")
            }
        }
        return telNumber
    }

    fun getRandomString(length: Int) : String {
        val charset = "ABCDEFGHIJKLMNOPQRSTUVWXTZabcdefghiklmnopqrstuvwxyz0123456789"
        return (1..length)
            .map { charset.random() }
            .joinToString("")
    }

    fun showToast(context: Context, msg : String) {
        Toast.makeText(context, msg, Toast.LENGTH_LONG).show()
    }

    fun getKeyHash(context: Context): String? {
        var packageInfo: PackageInfo? = null
        try {
            packageInfo = context.packageManager.getPackageInfo(context.packageName, PackageManager.GET_SIGNATURES)
        } catch (e: PackageManager.NameNotFoundException) {
            e.printStackTrace()
        }
        if (packageInfo == null) Timber.e("KeyHash:null")
        for (signature in packageInfo!!.signatures) {
            try {
                val md: MessageDigest = MessageDigest.getInstance("SHA")
                md.update(signature.toByteArray())
                return Base64.encodeToString(md.digest(), Base64.NO_WRAP)
                Timber.d(Base64.encodeToString(md.digest(), Base64.DEFAULT))
            } catch (e: NoSuchAlgorithmException) {
                Timber.e("Unable to get MessageDigest. signature=$signature")
            }
        }
        return null
    }

    fun getCurrentDate(): String {
        var strNow = ""
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
//            val currentDate: LocalDate = LocalDate.now()
//            strNow = currentDate.format(DateTimeFormatter.ofPattern("yyyyMMdd"))
            val currentDate: LocalDateTime = LocalDateTime.now()
            strNow = currentDate.format(DateTimeFormatter.ofPattern("yyyyMMddHHmmss"))
        } else {
            val now : Long = System.currentTimeMillis()
            val date = Date(now)
            val sdf = SimpleDateFormat("yyyyMMddHHmmss")
            strNow = sdf.format(date)
        }
        return strNow
    }

    fun uniToKor(uni: String): String {
        val result = StringBuffer()
        var i = 0
        while (i < uni.length) {
            if (uni[i] == '\\' && uni[i + 1] == 'u') {
                val c = uni.substring(i + 2, i + 6).toInt(16).toChar()
                result.append(c)
                i += 5
            } else {
                result.append(uni[i])
            }
            i++
        }
        return result.toString()
    }

    fun lpad(str : String, length : Int, prefix : String) : String {
        try {
            val sb = StringBuilder()
            for (i in str.length until length) {
                sb.append(prefix)
            }
            sb.append(str)
            return sb.toString()
        } catch (e: Exception) {
            return ""
        }
    }

    fun rpad(str : String, length : Int, prefix : String) : String {
        try {
            val sb = StringBuilder()
            sb.append(str)
            for (i in str.length until length) {
                sb.append(prefix)
            }
            return sb.toString()
        } catch (e: Exception) {
            return ""
        }
    }

}