package com.noorija.secret.guard.view

import android.content.Context
import android.content.Intent
import android.net.ConnectivityManager
import android.net.ConnectivityManager.NetworkCallback
import android.net.Network
import android.net.NetworkRequest
import android.os.Build
import android.os.Bundle
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.webkit.CookieSyncManager
import android.widget.Toast
import androidx.annotation.RequiresApi
import androidx.appcompat.app.AlertDialog
import androidx.appcompat.app.AppCompatActivity
import androidx.databinding.DataBindingUtil
import androidx.databinding.ViewDataBinding
import com.noorija.secret.guard.SecretGuardApp
import com.noorija.secret.guard.databinding.DlgCustomPopupBinding
import com.noorija.secret.guard.listener.NetworkStateReceiverListener
import com.noorija.secret.guard.text.Command
import com.noorija.secret.guard.text.DialogResultType
import com.noorija.secret.guard.text.DialogType
import com.noorija.secret.guard.util.PreferencesUtil
import com.noorija.secret.guard.viewmodel.BaseViewModel
import java.io.Serializable

//abstract class BaseActivity<T : ViewDataBinding, R : BaseViewModel<*>> : AppCompatActivity()
abstract class BaseActivity : AppCompatActivity()
    , NetworkStateReceiverListener {

    protected var mContext: Context? = null

//    lateinit var binding: T
//    abstract val layoutResourceId: Int
//    abstract val viewModel: R

    abstract fun initDataBinding()
    abstract fun initAfterBinding()

    private var networkCallback: NetworkCallback? = null
    private var connectivityManager: ConnectivityManager? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        mContext = this

//        binding = DataBindingUtil.setContentView(this, layoutResourceId)
//        binding.lifecycleOwner = this

        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.LOLLIPOP) {
            CookieSyncManager.createInstance(this)
        }

        connectivityManager = getSystemService(CONNECTIVITY_SERVICE) as ConnectivityManager
        if (this is NetworkStateReceiverListener) registerConnectivityMonitoring(this)

        initDataBinding()
        initAfterBinding()
    }

    override fun onPause() {
        super.onPause()
        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.LOLLIPOP) {
            CookieSyncManager.getInstance().stopSync()
        }
    }

    override fun onResume() {
        super.onResume()
        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.LOLLIPOP) {
            CookieSyncManager.getInstance().startSync()
        }
        if(SecretGuardApp.instance!!.isAppRestart()) {
//            val i = baseContext.packageManager.getLaunchIntentForPackage(baseContext.packageName)
//            i!!.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP)
//            i.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK)
//            i.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
//            startActivity(i)
//            finish()

//            start<SignInActivity>()
        }
    }

    @RequiresApi(api = Build.VERSION_CODES.LOLLIPOP)
    open fun registerConnectivityMonitoring(listener: NetworkStateReceiverListener) {
        val networkCallback: NetworkCallback = object : NetworkCallback() {
            override fun onAvailable(network: Network) {
                listener.networkConnectivityChanged(true)
            }

            override fun onLost(network: Network) {
                listener.networkConnectivityChanged(false)
            }
        }
        this.networkCallback = networkCallback
        connectivityManager!!.registerNetworkCallback(
            NetworkRequest.Builder().build(),
            networkCallback
        )
    }

    open fun <T> start(clazz: Class<T>) {
        val intent = Intent(this, clazz)
        startActivity(intent)
    }

    fun <T: Serializable> Intent.intentSerializable(key: String, clazz: Class<T>): T? {
        return if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
            this.getSerializableExtra(key, clazz)
        } else {
            this.getSerializableExtra(key) as T?
        }
    }

    protected fun showToast(message: String?, gravity: Int = Gravity.BOTTOM) {
        val toast = Toast.makeText(this, message, Toast.LENGTH_SHORT)
        toast?.setGravity(gravity, 0, 0)
        toast?.show()
    }

    open fun showCustomDialog(
        id: Int,
        message: String,
        dialogType: DialogType = DialogType.ALERT,
        title: String = "",
        btnLeftText: String = "",
        btnRightText: String = ""
    ) {
        val dialogView = DlgCustomPopupBinding.inflate(LayoutInflater.from(this))
        val builder = AlertDialog.Builder(this).setView(dialogView.root)
        val alertDialog = builder.create()
        alertDialog.show()

        if (title.isEmpty())
            dialogView.title.visibility = View.GONE
        else
            dialogView.title.text = title
        dialogView.message.text = message
        if (dialogType == DialogType.ALERT) {
            dialogView.btnCancel.visibility = View.GONE
        } else {
            if (btnLeftText.isNotEmpty()) dialogView.btnCancel.text = btnLeftText
        }
        if (btnRightText.isNotEmpty()) dialogView.btnConfirm.text = btnRightText
        dialogView.btnConfirm.setOnClickListener {
            alertDialog.dismiss()
            onDialogResult(id, DialogResultType.CONFIRM)
        }
        dialogView.btnCancel.setOnClickListener {
            alertDialog.dismiss()
            onDialogResult(id, DialogResultType.CANCEL)
        }
    }

    protected open fun onDialogResult(id: Int, result: DialogResultType) { }

}