package com.noorija.secret.guard.view

import android.app.Activity
import android.content.Intent
import android.content.pm.PackageManager
import android.os.Build
import android.os.Bundle
import android.view.View
import androidx.databinding.DataBindingUtil
import com.journeyapps.barcodescanner.CaptureManager
import com.journeyapps.barcodescanner.DecoratedBarcodeView
import com.noorija.secret.guard.R
import com.noorija.secret.guard.databinding.ActCustomScannerBinding
import timber.log.Timber

class CustomScannerActivity : Activity(), DecoratedBarcodeView.TorchListener {
    private var viewBinding: ActCustomScannerBinding? = null
    private var capture: CaptureManager? = null
    private var switchFlashlightButtonCheck: Boolean? = null
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        val view = window.decorView
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            if (view != null) {
                view.systemUiVisibility = View.SYSTEM_UI_FLAG_LIGHT_STATUS_BAR
            }
        }
        doFullScreen()
        viewBinding = DataBindingUtil.setContentView(this, R.layout.act_custom_scanner)
        switchFlashlightButtonCheck = true
        if (!hasFlash()) {
            viewBinding?.switchFlashlight?.setVisibility(View.GONE)
        }
        viewBinding?.zxingBarcodeScanner?.setTorchListener(this)
        capture = CaptureManager(this, viewBinding?.zxingBarcodeScanner)
        capture!!.initializeFromIntent(intent, savedInstanceState)
        capture!!.decode()
    }

    private fun doFullScreen() {
        val decorView = window.decorView
        decorView.systemUiVisibility = View.SYSTEM_UI_FLAG_IMMERSIVE or
                View.SYSTEM_UI_FLAG_LAYOUT_STABLE or
                View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION or
                View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN or
                View.SYSTEM_UI_FLAG_HIDE_NAVIGATION or
                View.SYSTEM_UI_FLAG_FULLSCREEN
    }

    override fun onResume() {
        super.onResume()
        capture!!.onResume()
    }

    override fun onPause() {
        super.onPause()
        capture!!.onPause()
    }

    override fun onDestroy() {
        super.onDestroy()
        capture!!.onDestroy()
    }

    override fun finish() {
        super.finish()
        overridePendingTransition(R.anim.anim_slide_from_left, R.anim.anim_slide_to_right)
    }

    override fun startActivity(intent: Intent) {
        super.startActivity(intent)
        overridePendingTransition(R.anim.anim_slide_from_right, R.anim.anim_slide_to_left)
    }

    override fun onSaveInstanceState(outState: Bundle) {
        super.onSaveInstanceState(outState)
        capture!!.onSaveInstanceState(outState)
    }

    override fun onBackPressed() {
        Timber.d("onBackPressed ===> ")
        super.onBackPressed()
    }

    fun onClosed(view: View?) {
        onBackPressed()
    }

    fun switchFlashlight(view: View?) {
        if (switchFlashlightButtonCheck!!) {
            viewBinding?.zxingBarcodeScanner?.setTorchOn()
        } else {
            viewBinding?.zxingBarcodeScanner?.setTorchOff()
        }
    }

    private fun hasFlash(): Boolean {
        return applicationContext.packageManager
            .hasSystemFeature(PackageManager.FEATURE_CAMERA_FLASH)
    }

    override fun onTorchOn() {
        //switchFlashlightButton.setImageResource(R.drawable.ic_flash_on_white_36dp);
        switchFlashlightButtonCheck = false
    }

    override fun onTorchOff() {
        //switchFlashlightButton.setImageResource(R.drawable.ic_flash_off_white_36dp);
        switchFlashlightButtonCheck = true
    }
}
