package com.noorija.secret.guard.view

import android.content.Intent
import android.net.Uri
import android.os.Build
import android.os.Bundle
import androidx.activity.viewModels
import androidx.annotation.RequiresApi
import androidx.databinding.DataBindingUtil
import androidx.lifecycle.Observer
import com.noorija.secret.guard.R
import com.noorija.secret.guard.databinding.ActSplashBinding
import com.noorija.secret.guard.text.AppUpgradeType
import com.noorija.secret.guard.text.DialogType
import com.noorija.secret.guard.viewmodel.SplashViewModel
import com.noorija.secret.guard.text.DialogResultType
import com.noorija.secret.guard.text.PageView

class FirstRegistActivity : BaseActivity() {

    private lateinit var binding : ActSplashBinding
    private val viewModel : SplashViewModel by viewModels()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding = DataBindingUtil.setContentView(this, R.layout.act_splash)
        binding.viewModel = viewModel
        binding.lifecycleOwner = this
    }

    override fun initDataBinding() {
    }

    @RequiresApi(Build.VERSION_CODES.O)
    override fun initAfterBinding() {
        with(viewModel) {
            upgradeEvent.observe(this@FirstRegistActivity, Observer {
                when (it) {
                    AppUpgradeType.MAJOR -> {}
                    AppUpgradeType.MINER -> {}
                    else -> { start(MainActivity::class.java) }
                }
            })
            finishEvent.observe(this@FirstRegistActivity, Observer {
                when(it) {
                    PageView.TUTORIAL -> {
                        start(TutorialActivity::class.java)
                        finish()
                    }
                    PageView.MAIN -> {
//                        start(TutorialActivity::class.java)
                        start(MainActivity::class.java)
                        finish()
                    }
                    PageView.STORE -> { goStore() }
                    PageView.FINISH -> { finish() }
                    PageView.DIALOG -> {
                        showCustomDialog(0, getString(R.string.msg_new_version)
                            , DialogType.CONFIRM, "", getString(R.string.btn_later), getString(R.string.btn_upgrade))
                    }
                    else -> {}
                }
            })
        }
    }

    private fun goStore() {
        val intent = Intent(Intent.ACTION_VIEW)
        intent.addCategory(Intent.CATEGORY_DEFAULT)
        intent.setData(Uri.parse("market://details?id=com.oqpo.app"))
        startActivity(intent)
        finish()
    }

    override fun onDialogResult(id: Int, result: DialogResultType) {
        when (result) {
            DialogResultType.CANCEL -> {
                if(viewModel.sysVersion.upRequireInfo == "20") {
                    if(viewModel.sysVersion.tutoStart == "Y") {
                        start(TutorialActivity::class.java)
                    } else {
                        start(TutorialActivity::class.java)
//                        start(MainActivity::class.java)
                    }
                }
                finish()
            }
            else -> {
                goStore()
            }
        }
    }

    override fun networkConnectivityChanged(isAvailable: Boolean) {

    }
}