package com.noorija.secret.guard.view

import android.os.Bundle
import android.view.animation.Animation
import android.view.animation.AnimationUtils
import androidx.activity.viewModels
import androidx.databinding.DataBindingUtil
import androidx.lifecycle.Observer
import androidx.viewpager2.widget.ViewPager2
import androidx.viewpager2.widget.ViewPager2.OnPageChangeCallback
import com.noorija.secret.guard.R
import com.noorija.secret.guard.databinding.ActGuideBinding
import com.noorija.secret.guard.view.adapter.GuidePagerAdapter
import com.noorija.secret.guard.viewmodel.GuideViewModel


class GuideActivity : BaseActivity() {

    private lateinit var binding : ActGuideBinding
    private val viewModel : GuideViewModel by viewModels()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding = DataBindingUtil.setContentView(this, R.layout.act_guide)
        binding.viewModel = viewModel
        binding.lifecycleOwner = this
    }

    override fun initDataBinding() {
    }

    private fun createPager(data: List<String>) {
        val guidePager: ViewPager2 = binding.guidePager
        guidePager.setAdapter(GuidePagerAdapter(data))
        guidePager.registerOnPageChangeCallback(object : OnPageChangeCallback() {
            override fun onPageSelected(position: Int) {
                super.onPageSelected(position)
                viewModel.isBottomVisibility.value = position == data.size - 1
                if (position == data.size - 1) {
                    val slowlyAppear: Animation =
                        AnimationUtils.loadAnimation(mContext, com.noorija.secret.guard.R.anim.anim_fade_in)
                    binding.btnStart.setAnimation(slowlyAppear)
                } else {
                }
            }
        })
        binding.indicator.setViewPager2(guidePager)
        binding.guidePager.setCurrentItem(0, false)
    }

    override fun initAfterBinding() {
        with(viewModel) {
            startEvent.observe(this@GuideActivity, Observer {
                start(MainActivity::class.java)
                finish()
            })
            guideEvent.observe(this@GuideActivity, Observer {
                createPager(it)
            })
        }
    }

    override fun networkConnectivityChanged(isAvailable: Boolean) {

    }
}