package com.noorija.secret.guard.view

import android.annotation.SuppressLint
import android.content.Intent
import android.os.Build
import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.os.ResultReceiver
import android.view.LayoutInflater
import android.view.View
import android.widget.TextView
import androidx.activity.OnBackPressedCallback
import androidx.activity.viewModels
import androidx.core.content.ContextCompat
import androidx.databinding.DataBindingUtil
import androidx.fragment.app.Fragment
import androidx.lifecycle.Observer
import com.google.android.material.tabs.TabLayout
import com.google.gson.Gson
import com.noorija.secret.guard.R
import com.noorija.secret.guard.databinding.ActMainBinding
import com.noorija.secret.guard.rest.RoomIdInfo
import com.noorija.secret.guard.service.FindCallService
import com.noorija.secret.guard.text.FloatingMenuType
import com.noorija.secret.guard.util.BackPressCloseHandler
import com.noorija.secret.guard.view.adapter.TabsMainAdapter
import com.noorija.secret.guard.view.fragment.HomeFragment
import com.noorija.secret.guard.view.fragment.MenuFragment
import com.noorija.secret.guard.view.fragment.QrcodeFragment
import com.noorija.secret.guard.view.fragment.SettingFragment
import com.noorija.secret.guard.viewmodel.MainViewModel
import timber.log.Timber


class MainActivity : BaseActivity() {
    companion object {
        const val ID_HOME_FRAGMENT = "100"
        const val ID_QR_CCODE_FRAGMENT = "200"
        const val ID_SETTING_FRAGMENT = "300"
        const val ID_MENU_FRAGMENT = "400"
        const val CMD_FIND_CALL_RECEIVER = "FIND_CALL_RECEIVER"
        const val CMD_ROOM_ID_LIST = "ROOM_ID_LIST"
    }
    private lateinit var binding : ActMainBinding
    private val viewModel : MainViewModel by viewModels()

    private val backPressCloseHandler = BackPressCloseHandler(this)
    private val backPressedCallback = object : OnBackPressedCallback(true) {
        override fun handleOnBackPressed() {
            backPressCloseHandler.onBackPressed()
        }
    }

    private var isFabOpen = false

    override fun initDataBinding() {
        binding = DataBindingUtil.setContentView(this, R.layout.act_main)
        binding.viewModel = viewModel
        binding.lifecycleOwner = this

        this.onBackPressedDispatcher.addCallback(this, backPressedCallback)
        createTab()
//        initNavigationBar()
    }

    override fun initAfterBinding() {
        with(viewModel) {
            floatingButtonEvent.observe(this@MainActivity, Observer {
                if(it) {
                    binding.fabMain.setImageDrawable(ContextCompat.getDrawable(applicationContext, R.drawable.ico_floating_close))
                } else {
                    binding.fabMain.setImageDrawable(ContextCompat.getDrawable(applicationContext, R.drawable.ico_floating_menu))
                }
            })
            floatingMenuEvent.observe(this@MainActivity, Observer {
                when (it) {
                    FloatingMenuType.MENU -> {}
                    FloatingMenuType.SET -> { startActivity(Intent(this@MainActivity, PreferencesActivity::class.java)) }
                    FloatingMenuType.FAQ -> { startActivity(Intent(this@MainActivity, FaqListActivity::class.java)) }
                    FloatingMenuType.ASK -> { startActivity(Intent(this@MainActivity, AskListActivity::class.java)) }
                    FloatingMenuType.NOTICE -> { startActivity(Intent(this@MainActivity, NoticeListActivity::class.java)) }
                    FloatingMenuType.MANUAL -> { startActivity(Intent(this@MainActivity, ManualActivity::class.java)) }
                    FloatingMenuType.QR -> {
                        startActivity(Intent(this@MainActivity, ScannerActivity::class.java).apply {
                            putExtra("CallType", "WebRTC")
                        })
                    }
                    else -> {}
                }
            })
            mainEvent.observe(this@MainActivity, Observer {
                startFindCallService(it)
            })
        }
    }

    private fun createTab() {
        val tabs = binding.tabs
        val viewPager = binding.viewPager
        tabs.addTab(
            tabs.newTab().setCustomView(createTabView(getString(R.string.tab_title_my_qrcode)))
        )
        tabs.addTab(
            tabs.newTab().setCustomView(createTabView(getString(R.string.tab_title_call_history)))
        )
        tabs.addTab(
            tabs.newTab().setCustomView(createTabView(getString(R.string.tab_title_public_reg)))
        )

        val adapter = TabsMainAdapter(supportFragmentManager, tabs!!.tabCount)
        viewPager.adapter = adapter
        viewPager.addOnPageChangeListener(TabLayout.TabLayoutOnPageChangeListener(tabs))

        tabs.addOnTabSelectedListener(object : TabLayout.OnTabSelectedListener {
            override fun onTabSelected(tab: TabLayout.Tab) {
                if(tab.position == 0) {
                } else {
                }
                viewPager.currentItem = tab.position
                setTabSelected(tab.position)
            }

            override fun onTabUnselected(tab: TabLayout.Tab) {
            }

            override fun onTabReselected(tab: TabLayout.Tab) {
            }
        })
        setTabSelected(0)
    }

    @SuppressLint("MissingInflatedId")
    private fun createTabView(tabName: String) : View {
        val tabView: View = LayoutInflater.from(applicationContext).inflate(R.layout.tab_act_main, null)
        val tabTitle: TextView = tabView.findViewById(R.id.tvTabTitle)
        tabTitle.text = tabName
        return tabView
    }

    private fun setTabSelected(idx: Int) {
        val tabs = binding.tabs
        for(i in 0 until tabs.tabCount) {
            val tabView = tabs.getTabAt(i)?.view
            val tabTitle: TextView = tabView?.findViewById(R.id.tvTabTitle)!!
            val tabInfo: TextView = tabView?.findViewById(R.id.tvTabInfo)!!
            if(i == idx) {
                tabTitle.setTextColor(ContextCompat.getColor(applicationContext, R.color.color_f9ac19))
                tabInfo.setTextColor(ContextCompat.getColor(applicationContext, R.color.color_f9ac19))
            } else {
                tabTitle.setTextColor(ContextCompat.getColor(applicationContext, R.color.color_3c3c3c))
                tabInfo.setTextColor(ContextCompat.getColor(applicationContext, R.color.color_3c3c3c))
            }
        }
    }

    override fun onBackPressed() {
//        if (isHome()) {
//            backPressCloseHandler.onBackPressed()
//        } else {
//            navigateToHome()
//        }
        super.onBackPressed()
    }

//    private fun toggleFab() {
//        if(isFabOpen) {
//            ObjectAnimator.ofFloat(binding.fabSet, "translationY", 0f).apply { start() }
//            ObjectAnimator.ofFloat(binding.fabFaq, "translationY", 0f).apply { start() }
//            ObjectAnimator.ofFloat(binding.fabInquiry, "translationY", 0f).apply { start() }
//            ObjectAnimator.ofFloat(binding.fabNotice, "translationY", 0f).apply { start() }
//            ObjectAnimator.ofFloat(binding.fabManual, "translationY", 0f).apply { start() }
//            ObjectAnimator.ofFloat(binding.fabManual, "translationY", 0f).apply { start() }
//            binding.fabMain.setImageResource(R.drawable.ico_floating_menu)
//        } else {
//            ObjectAnimator.ofFloat(binding.fabSet, "translationY", -200f).apply { start() }
//            ObjectAnimator.ofFloat(binding.fabFaq, "translationY", -2000f).apply { start() }
//            ObjectAnimator.ofFloat(binding.fabInquiry, "translationY", -2000f).apply { start() }
//            ObjectAnimator.ofFloat(binding.fabNotice, "translationY", -2000f).apply { start() }
//            ObjectAnimator.ofFloat(binding.fabManual, "translationY", -2000f).apply { start() }
//            ObjectAnimator.ofFloat(binding.fabManual, "translationY", -2000f).apply { start() }
//            binding.fabMain.setImageResource(R.drawable.ico_floating_close)
//        }
//        isFabOpen = !isFabOpen
//    }




//    private fun isHome(): Boolean {
//        return binding.bottomNavigationView.menu.findItem(R.id.navigation_home).isChecked
//    }
//
//    private fun navigateToHome() {
//        val homeItem = binding.bottomNavigationView.menu.findItem(R.id.navigation_home)
//        homeItem.isChecked = true
//        val stackName = homeItem.itemId.toString()
//        val stackCount = supportFragmentManager.backStackEntryCount
//        for (x in 1 until stackCount) supportFragmentManager.popBackStack()
//        supportFragmentManager.popBackStack(stackName, FragmentManager.POP_BACK_STACK_INCLUSIVE)
//    }
//
//    private fun initNavigationBar() {
//        binding.bottomNavigationView.run {
//            setOnItemSelectedListener { item ->
//                loadFragment(item.itemId)
//            }
//        }
//    }

    private fun loadFragment(itemId: Int): Boolean {
        val tag = itemId.toString()
        var fragment: Fragment? = null
        when (itemId) {
            R.id.navigation_home -> {
                clearStack()
                fragment = HomeFragment.newInstance()
            }
            R.id.navigation_qrcode -> {
                fragment = QrcodeFragment.newInstance()
            }
            R.id.navigation_setting -> {
                fragment = SettingFragment.newInstance()
            }
            R.id.navigation_menu -> {
                fragment = MenuFragment.newInstance()
            }
            else -> {
            }
        }

        if (fragment != null && !supportFragmentManager.isDestroyed) {
            supportFragmentManager
                .beginTransaction()
                .replace(R.id.container, fragment, tag)
                .addToBackStack(null)
                .commitAllowingStateLoss()
            //.commit()
        }
        return true
    }

    private fun clearStack() {
        val backStackEntry = supportFragmentManager.backStackEntryCount
        if (backStackEntry > 0) {
            for (i in 0 until backStackEntry) {
                supportFragmentManager.popBackStackImmediate()
            }
        }
        if (supportFragmentManager.fragments.size > 0) {
            supportFragmentManager.fragments.forEach {
                if (it != null) {
                    supportFragmentManager.beginTransaction().remove(it).commit()
                }
            }
        }
    }

    private fun moveFragment(fragment: Fragment, tag: String) {
        if (!supportFragmentManager.isDestroyed) {
            supportFragmentManager
                .beginTransaction()
                .replace(R.id.container, fragment, tag)
                .addToBackStack(null)
                .commitAllowingStateLoss()
            //.commit()
        }
    }

    private fun startFindCallService(roomIdList: List<RoomIdInfo>) {
        // 인텐트 생성
        val intent = Intent(this@MainActivity, FindCallService::class.java)

        // MyResultReceiver 객체를 생성하여 putExtra() 메소드를 사용하여 Intent의 extra로 전달합니다.
        intent.putExtra(CMD_FIND_CALL_RECEIVER, resultReceiver) //(2)
        intent.putExtra(CMD_ROOM_ID_LIST, Gson().toJson(roomIdList))

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            // 안드로이드 운영 체제에서 포그라운드 서비스를 시작하는 메서드
            applicationContext.startForegroundService(intent)
        } else {
            applicationContext.startService(intent)
        }
    }

    var handler: Handler = Handler(Looper.getMainLooper())

    private val resultReceiver: ResultReceiver = object : ResultReceiver(handler) {
        // (3)
        // 서비스에서 전달된 결과를 처리합니다.
        override fun onReceiveResult(resultCode: Int, resultData: Bundle?) {
            super.onReceiveResult(resultCode, resultData)

            // SYNC_COMPLETED CODE = 1
            if (resultCode == 1) {  // (4)
                showToast("ResultReceiver......")
            }
        }
    }

    override fun networkConnectivityChanged(isAvailable: Boolean) {

    }
}
