package com.noorija.secret.guard.view

import android.os.Build
import android.os.Bundle
import androidx.activity.viewModels
import androidx.databinding.DataBindingUtil
import androidx.lifecycle.Observer
import com.noorija.secret.guard.R
import com.noorija.secret.guard.databinding.ActMemberJoinBinding
import com.noorija.secret.guard.model.CasAgreeInfo
import com.noorija.secret.guard.rest.SaveCasInfo
import com.noorija.secret.guard.util.AndroidBug5497Workaround
import com.noorija.secret.guard.util.BackPressCloseHandler
import com.noorija.secret.guard.util.KeyboardVisibilityUtils
import com.noorija.secret.guard.viewmodel.MemberJoinViewModel

class MemberJoinActivity : BaseActivity() {
    private lateinit var binding : ActMemberJoinBinding
    private val viewModel : MemberJoinViewModel by viewModels()

    private lateinit var keyboardVisibilityUtils: KeyboardVisibilityUtils
    private val backPressCloseHandler = BackPressCloseHandler(this)

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
//        AndroidBug5497Workaround.assistActivity(this)
        binding = DataBindingUtil.setContentView(this, R.layout.act_member_join)
        binding.viewModel = viewModel
        binding.lifecycleOwner = this

        keyboardVisibilityUtils = KeyboardVisibilityUtils(window,
            onShowKeyboard = { keyboardHeight ->
                binding.scrollView.run {
                    smoothScrollTo(scrollX, scrollY + keyboardHeight)
                }
            })

        if(Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
            intent.getSerializableExtra("data", ArrayList::class.java)
                ?.let { viewModel.setCasAgreeList(it as ArrayList<SaveCasInfo>) }
        } else {
            intent.getSerializableExtra("data")
                ?.let { viewModel.setCasAgreeList(it as ArrayList<SaveCasInfo>) }
        }
    }

    override fun initDataBinding() {
    }

    override fun initAfterBinding() {
        with(viewModel) {
            memberJoinEvent.observe(this@MemberJoinActivity, Observer {
                when (it) {
                    is List<*> -> {
                    }

                    else -> {
                        start(MainActivity::class.java)
                        finish()
                    }
                }
            })
        }
    }

    override fun onBackPressed() {
        super.onBackPressed()
        backPressCloseHandler.onBackPressed()
    }

    override fun networkConnectivityChanged(isAvailable: Boolean) {

    }
}