package com.noorija.secret.guard.view

import android.content.Intent
import androidx.activity.viewModels
import androidx.databinding.DataBindingUtil
import androidx.lifecycle.Observer
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.noorija.secret.guard.R
import com.noorija.secret.guard.databinding.ActNoticeListBinding
import com.noorija.secret.guard.view.adapter.NoticeRecyclerAdapter
import com.noorija.secret.guard.viewmodel.NoticeListViewModel

class NoticeListActivity : BaseActivity() {

    private lateinit var binding : ActNoticeListBinding
    private val viewModel : NoticeListViewModel by viewModels()

    private lateinit var mRecyclerAdapter : NoticeRecyclerAdapter

    override fun initDataBinding() {
        binding = DataBindingUtil.setContentView(this, R.layout.act_notice_list)
        binding.viewModel = viewModel
        binding.lifecycleOwner = this

        setRecyclerView()
    }

    override fun initAfterBinding() {
        with(viewModel) {
            closeEvent.observe(this@NoticeListActivity, Observer {
//                val intent = Intent(this@PreferencesActivity, InfoRegistActivity::class.java)
//                startActivity(intent)
                finish()
            })
        }
    }

    private fun setRecyclerView() {
        mRecyclerAdapter = NoticeRecyclerAdapter(viewModel)
        binding.noticeRecyclerView.apply {
            adapter = mRecyclerAdapter
            layoutManager = LinearLayoutManager(this@NoticeListActivity, RecyclerView.VERTICAL, false)
        }

        binding.noticeRecyclerView.addOnScrollListener(object : RecyclerView.OnScrollListener() {
            override fun onScrollStateChanged(recyclerView: RecyclerView, newState: Int) {
                super.onScrollStateChanged(recyclerView, newState)
                val layoutManager = recyclerView.layoutManager as LinearLayoutManager
                val totalItemCount = layoutManager.itemCount
                val lastVisible = layoutManager.findLastCompletelyVisibleItemPosition()
                if (lastVisible >= totalItemCount - 1) {
                    viewModel.reqNoticeList()
                }
            }
        })
    }

    override fun networkConnectivityChanged(isAvailable: Boolean) {

    }
}