package com.noorija.secret.guard.view

import android.net.Uri
import android.view.View
import android.widget.AdapterView
import android.widget.ArrayAdapter
import androidx.activity.result.ActivityResultCallback
import androidx.activity.viewModels
import androidx.databinding.DataBindingUtil
import androidx.lifecycle.Observer
import com.noorija.secret.guard.R
import com.noorija.secret.guard.databinding.ActPreferencesBinding
import com.noorija.secret.guard.util.PickRingtone
import com.noorija.secret.guard.viewmodel.PreferencesViewModel
import timber.log.Timber


class PreferencesActivity : BaseActivity() {

    private lateinit var binding : ActPreferencesBinding
    private val viewModel : PreferencesViewModel by viewModels()

    override fun initDataBinding() {
        binding = DataBindingUtil.setContentView(this, R.layout.act_preferences)
        binding.viewModel = viewModel
        binding.lifecycleOwner = this

        setAlarmSpinner()
    }

    override fun initAfterBinding() {
        with(viewModel) {
            closeEvent.observe(this@PreferencesActivity, Observer {
//                val intent = Intent(this@PreferencesActivity, InfoRegistActivity::class.java)
//                startActivity(intent)
                finish()
            })
            ringtoneEvent.observe(this@PreferencesActivity, Observer {
//                val intent = Intent(RingtoneManager.ACTION_RINGTONE_PICKER)
//                intent.putExtra(RingtoneManager.EXTRA_RINGTONE_TITLE, "Select ringtone for notifications:")
//                intent.putExtra(RingtoneManager.EXTRA_RINGTONE_SHOW_SILENT, false)
//                intent.putExtra(RingtoneManager.EXTRA_RINGTONE_SHOW_DEFAULT, true)
//                intent.putExtra(RingtoneManager.EXTRA_RINGTONE_TYPE,RingtoneManager.TYPE_NOTIFICATION)
//                startActivityForResult(intent, 777)
                ringtoneLauncher.launch(1)
            })
        }
    }

    private val ringtoneLauncher = registerForActivityResult(PickRingtone()) {
        //handle the returned uri
        ActivityResultCallback<Uri> registerForActivityResult@{ result: Uri? ->
            if (result == null) {
                return@registerForActivityResult
            }
            val uri = PickRingtone.noneToNull(result)
            Timber.d("ringtoneLauncher ================> %s", uri)
            Timber.d("ringtoneLauncher ================> %s", uri.toString())
        }
    }

    private fun setAlarmSpinner() {
        val arr = resources.getStringArray(R.array.arrAlarmInterval)
        var spinnerAdapter =
            ArrayAdapter(this@PreferencesActivity, R.layout.item_spinner_dropdown, arr)
        binding.spAlarmInterval.adapter = spinnerAdapter
        binding.spAlarmInterval.onItemSelectedListener = object : AdapterView.OnItemSelectedListener {
            override fun onItemSelected(parent: AdapterView<*>, view: View?, pos: Int, id: Long) {
                viewModel.setAlarmInterval(arr[pos])
            }

            override fun onNothingSelected(p0: AdapterView<*>?) {
            }
        }
    }

    override fun networkConnectivityChanged(isAvailable: Boolean) {

    }
}