package com.noorija.secret.guard.view

import android.content.ComponentName
import android.content.Context
import android.content.Intent
import android.content.ServiceConnection
import android.os.IBinder
import androidx.activity.viewModels
import androidx.databinding.DataBindingUtil
import androidx.lifecycle.Observer
import com.noorija.secret.guard.R
import com.noorija.secret.guard.databinding.ActRcvMsgBinding
import com.noorija.secret.guard.service.FindCallService
import com.noorija.secret.guard.viewmodel.RcvMsgViewModel
import timber.log.Timber

class RcvMsgActivity : BaseActivity() {

    private lateinit var binding : ActRcvMsgBinding
    private val viewModel : RcvMsgViewModel by viewModels()

    private var isService = false
    private lateinit var nickNm: String
    private lateinit var strPayload: String

    override fun initDataBinding() {
        binding = DataBindingUtil.setContentView(this, R.layout.act_rcv_msg)
        binding.viewModel = viewModel
        binding.lifecycleOwner = this

        nickNm = intent?.getStringExtra("StompNickName").toString()
        strPayload = intent?.getStringExtra("StompPayload").toString()
        Timber.d("nickNm >>>>>>>>>>>>>>>>>>>>>>> $nickNm")
        Timber.d("strPayload >>>>>>>>>>>>>>>>>>>>>>> $strPayload")
        viewModel.init(nickNm, strPayload)

        val intent = Intent(applicationContext, FindCallService::class.java)
        bindService(intent , serviceConnection, Context.BIND_AUTO_CREATE)
    }

    override fun initAfterBinding() {
        with(viewModel) {
            sendMessageEvent.observe(this@RcvMsgActivity, Observer {
                Timber.d("sendMessageEvent >>>>>>>>>>>>>>>>>>>>>> $isService : $strPayload")
                if(isService) {
                    stompService?.bindMessage(0, it as String, strPayload)
                    finish()
                }
            })
            sendCallEvent.observe(this@RcvMsgActivity, Observer {
            })
            closeEvent.observe(this@RcvMsgActivity, Observer {
                finish()
            })
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        if(isService) {
            unbindService(serviceConnection)
            isService = false
        }
    }

    var stompService: FindCallService? = null
    private val serviceConnection = object : ServiceConnection {
        override fun onServiceConnected(p0: ComponentName?, p1: IBinder?) {
            Timber.d("=======================> ServiceConnected")
            isService = true
            stompService = (p1 as FindCallService.LocalBinder).getService()
        }

        override fun onServiceDisconnected(p0: ComponentName?) {
            Timber.d("=======================> ServiceDisconnected")
            isService = false
            stompService = null
        }
    }

    override fun networkConnectivityChanged(isAvailable: Boolean) {

    }
}