package com.noorija.secret.guard.view

import android.content.ActivityNotFoundException
import android.content.Context
import android.content.Intent
import android.net.Uri
import android.os.Build
import android.view.View
import android.webkit.*
import androidx.activity.viewModels
import androidx.databinding.DataBindingUtil
import com.noorija.secret.guard.R
import com.noorija.secret.guard.databinding.ActRtcWebviewBinding
import com.noorija.secret.guard.viewmodel.RtcWebViewViewModel
import timber.log.Timber
import java.net.URISyntaxException

class RtcWebViewActivity : BaseActivity() {
    companion object {
        const val ACTIVITY_WEBVIEW_TITLE            = "SecretGuardWebRtc"
        const val ACTIVITY_RESULT_SECRET_GUARD      = "ActivityResultSecretGuard"
        const val WEB_RTC_SERVER_URL                = "https://kosk.co.kr/scgdCall/callview.html?qrNo=QR2409010000053"
    }

    private lateinit var binding : ActRtcWebviewBinding
    private val viewModel : RtcWebViewViewModel by viewModels()

    private val webRtcURL by lazy {
        intent.getStringExtra("RTC_URL")
    }

    override fun initDataBinding() {
        binding = DataBindingUtil.setContentView(this, R.layout.act_rtc_webview)
        binding.viewModel = viewModel
        binding.lifecycleOwner = this

        Timber.d("WEB_RTC_SERVER_URL ==============> %s", webRtcURL)
        initWebView(WEB_RTC_SERVER_URL)
//        initWebView(webRtcURL)
    }

    override fun initAfterBinding() {
    }

    private fun initWebView(webRtcURL: String?) {
        val mWebView = binding.mWebView

        mWebView.settings.javaScriptEnabled = true
        mWebView.settings.domStorageEnabled = true

        if ( Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP ) {
            mWebView.settings.mixedContentMode = WebSettings.MIXED_CONTENT_ALWAYS_ALLOW
            val cookieManager = CookieManager.getInstance()
            cookieManager.setAcceptCookie(true)
            cookieManager.setAcceptThirdPartyCookies(mWebView, true)
        }

        mWebView.webChromeClient = SecretGuardWebChromeClient()
        mWebView.webViewClient = SecretGuardWebViewClient()
        mWebView.addJavascriptInterface(JavaScriptInterface(this), ACTIVITY_WEBVIEW_TITLE)
        mWebView.loadUrl(webRtcURL!!)
//        mWebView.loadUrl(postData.toString())

    }

    inner class SecretGuardWebChromeClient : WebChromeClient() {

    }

    inner class SecretGuardWebViewClient : WebViewClient() {
        override fun shouldOverrideUrlLoading(view: WebView, url: String): Boolean {
            if (url.isNotBlank() && url != "about:blank") {
                if (url.startsWith("http://") || url.startsWith("https://")) {
                    if (url.contains("http://market.android.com") ||
                        url.contains("http://m.ahnlab.com/kr/site/download") ||
                        url.endsWith(".apk")) {
                        return urlSchemeIntent(view, url)
                    } else {
                        view.loadUrl(url)
                    }
                } else return if (url.startsWith("mailto:")) {
                    false
                } else if (url.startsWith("tel:")) {
                    false
                } else {
                    urlSchemeIntent(view, url)
                }
            }
            return true
        }
        override fun onPageFinished(view: WebView, url: String) {
            binding.layoutProgress.visibility = View.GONE
        }
    }

    fun urlSchemeIntent(view: WebView, url: String): Boolean {
        if (url.startsWith("intent")) {
            var intent: Intent? = null
            intent = try {
                Intent.parseUri(url, Intent.URI_INTENT_SCHEME)
            } catch (ex: URISyntaxException) {
                return false
            }

            if (packageManager.resolveActivity(intent!!, 0) == null) {
                val packagename = intent.getPackage()
                if (packagename != null) {
                    startActivity(Intent(Intent.ACTION_VIEW, Uri.parse("market://search?q=pname:$packagename")))
                    return true
                }
            }
            intent = Intent(Intent.ACTION_VIEW, Uri.parse(intent.dataString))
            try {
                startActivity(intent)
            } catch (e: ActivityNotFoundException) {
                return false
            }
        } else {
            try {
                startActivity(Intent(Intent.ACTION_VIEW, Uri.parse(url)))
            } catch (e: Exception) {
                showToast("해당 어플을 설치해 주세요.")
            }
        }
        return true
    }

    private inner class JavaScriptInterface(context: Context) {
        private val context = context
        @JavascriptInterface
        fun orderComplete(result: String) {
            //Timber.d("GalaxiaPay orderComplete >>> $result")
            val intent = Intent()
            if (result != null) {

            } else {
                //setResult(Activity.RESULT_CANCELED)
            }
        }
    }

    override fun networkConnectivityChanged(isAvailable: Boolean) {
    }
}