package com.noorija.secret.guard.view

import android.content.Intent
import android.os.Bundle
import androidx.activity.viewModels
import androidx.databinding.DataBindingUtil
import androidx.lifecycle.Observer
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import androidx.viewpager2.widget.ViewPager2
import androidx.viewpager2.widget.ViewPager2.OnPageChangeCallback
import com.noorija.secret.guard.R
import com.noorija.secret.guard.databinding.ActTutorialBinding
import com.noorija.secret.guard.rest.CasMngInfo
import com.noorija.secret.guard.view.adapter.CasRecyclerAdapter
import com.noorija.secret.guard.view.adapter.GuidePagerAdapter
import com.noorija.secret.guard.viewmodel.TutorialViewModel

class TutorialActivity : BaseActivity() {

    private lateinit var binding : ActTutorialBinding
    private val viewModel : TutorialViewModel by viewModels()

    private lateinit var mRecyclerAdapter : CasRecyclerAdapter

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding = DataBindingUtil.setContentView(this, R.layout.act_tutorial)
        binding.viewModel = viewModel
        binding.lifecycleOwner = this
        setRecyclerView()
    }

    override fun initDataBinding() {
    }

    override fun initAfterBinding() {
        with(viewModel) {
            tutorialEvent.observe(this@TutorialActivity, Observer {
                when (it) {
                    is List<*> -> {
                        createPager(it as List<String>)
                    }
                    is CasMngInfo -> {
                    }
                    else -> {
                        start(InfoRegistActivity::class.java)
                        finish()
                    }
                }
            })
            joinEvent.observe(this@TutorialActivity, Observer {
                val intent = Intent(this@TutorialActivity, MemberJoinActivity::class.java)
                intent.putParcelableArrayListExtra("data", it)
                startActivity(intent)
                finish()
            })
        }
    }

    private fun createPager(data: List<String>) {
        val tutorialPager: ViewPager2 = binding.tutorialPager
        tutorialPager.setAdapter(GuidePagerAdapter(data))
        tutorialPager.registerOnPageChangeCallback(object : OnPageChangeCallback() {
            override fun onPageSelected(position: Int) {
                super.onPageSelected(position)
                viewModel.bottomVisibility.set(position == data.size - 1)
                if (viewModel.bottomVisibility.get()) {
                    viewModel.getCas()
//                    val slowlyAppear: Animation =
//                        AnimationUtils.loadAnimation(mContext, R.anim.anim_fade_in)
//                    binding.btnStart.setAnimation(slowlyAppear)
                }
            }
        })
        binding.indicator.setViewPager2(tutorialPager)
        binding.tutorialPager.setCurrentItem(0, false)
    }

    private fun setRecyclerView() {
        mRecyclerAdapter = CasRecyclerAdapter(viewModel)
        binding.casRecyclerView.apply {
            adapter = mRecyclerAdapter
            layoutManager = LinearLayoutManager(this@TutorialActivity, RecyclerView.VERTICAL, false)
        }
    }

    override fun networkConnectivityChanged(isAvailable: Boolean) {

    }
}