package com.noorija.secret.guard.view.adapter

import androidx.recyclerview.widget.RecyclerView

import android.util.SparseBooleanArray
import android.view.View
import android.view.ViewGroup
import java.util.*

open class BaseRecyclerAdapter<T> : RecyclerView.Adapter<BaseRecyclerAdapter.ViewHolder<T>>() {

    companion object {
        const val VIEW_TYPE_EMPTY = 0
        const val VIEW_TYPE_NORMAL = 1
    }

    var curPossion = 0
    val mSelectedItems = SparseBooleanArray(0)

    private var onItemClickListener: OnItemClickListener<T>? = null

    interface OnItemClickListener<T> {
        fun onItemClick(view: View, item: T, position: Int)
    }

    fun setOnItemClickListener(listener: (View, T, Int) -> Unit) {
        this.onItemClickListener = object : OnItemClickListener<T> {
            override fun onItemClick(view: View, item: T, position: Int) {
                listener(view, item, position)
            }
        }
    }

    fun getOnItemClickListener(): OnItemClickListener<T>? {
        return onItemClickListener
    }

    private var onItemChangeListener: OnItemChangeListener<T>? = null

    interface OnItemChangeListener<T> {
        fun onItemChange(item: T)
    }

    fun setOnItemChangeListener(listener: (T) -> Unit) {
        this.onItemChangeListener = object : OnItemChangeListener<T> {
            override fun onItemChange(item: T) {
                listener(item)
            }
        }
    }

    fun getOnItemChangeListener(): OnItemChangeListener<T>? {
        return onItemChangeListener
    }

    override fun getItemId(position: Int): Long {
        return position.toLong()
    }


    private var items = Collections.emptyList<T>()

    override fun getItemCount(): Int {
        return if (items.isEmpty()) 1 else items.size
    }

    override fun getItemViewType(position: Int): Int {
        return if (items.isEmpty()) VIEW_TYPE_EMPTY else VIEW_TYPE_NORMAL
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolder<T> {
        TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
    }

    override fun onBindViewHolder(holder: ViewHolder<T>, position: Int) {
        if (getItemViewType(position) == VIEW_TYPE_NORMAL)
            holder.populate(items[position], position)
    }

    override fun onBindViewHolder(holder: ViewHolder<T>, position: Int, payloads: List<Any>) {
        if (getItemViewType(position) == VIEW_TYPE_NORMAL) {
            if (payloads.isEmpty())
                holder.populate(items[position], position)
            else
                holder.populate(items[position], position, payloads)
        }
    }

    fun updateList(itemList: List<T>) {
        items = itemList
        notifyDataSetChanged()
    }

    fun getItems(): List<T> {
        return items
    }

    fun getItem(pos: Int): T {
        return items[pos]
    }

    fun addItem(item: T, pos:Int = -1) {
        if(pos == -1)
            items.add(item)
        else
            items.add(pos, item)
    }

    private fun removeItem(item: T) {
        items.remove(item)
    }

    fun removeItem(pos: Int) {
        items.removeAt(pos)
    }

    fun moveItem(from: Int, to: Int) {
        val item = items[from]
        removeItem(item)
        addItem(item, to)
        notifyItemMoved(from, to)
    }

    open class ViewHolder<T>(
        itemView: View,
        var baseRecyclerAdapter: BaseRecyclerAdapter<T>
    ) : RecyclerView.ViewHolder(itemView), View.OnClickListener {

        init {
            itemView.setOnClickListener(this)
        }

        open fun populate(item: T, position: Int) {}

        open fun populate(item: T, position: Int, payloads: List<Any>) {}

        override fun onClick(v: View) {
            if (itemViewType != VIEW_TYPE_EMPTY) {
                baseRecyclerAdapter.onItemClickListener?.onItemClick(
                    v,
                    baseRecyclerAdapter.items[adapterPosition],
                    adapterPosition
                )
                baseRecyclerAdapter.curPossion = adapterPosition
            }
        }

    }

}