package com.noorija.secret.guard.view.adapter

import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.LinearLayout
import com.noorija.secret.guard.databinding.ItemEmptyBinding
import com.noorija.secret.guard.databinding.ItemFaqListBinding
import com.noorija.secret.guard.rest.FaqInfo
import com.noorija.secret.guard.util.ToggleAnimation
import com.noorija.secret.guard.viewmodel.FaqListViewModel

class FaqRecyclerAdapter(val viewmodel : FaqListViewModel): BaseRecyclerAdapter<FaqInfo>() {

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolder<FaqInfo> {
        return if(getItemViewType(viewType) == VIEW_TYPE_NORMAL) {
            val binding = ItemFaqListBinding.inflate(LayoutInflater.from(parent.context), parent, false)
            RecyclerViewHolder(
                binding,
                this,
                viewmodel
            )
        } else {
            val binding = ItemEmptyBinding.inflate(LayoutInflater.from(parent.context), parent, false)
            EmptyViewHolder(
                binding,
                this
            )
        }
    }

    class RecyclerViewHolder(
        private var itemBinding: ItemFaqListBinding,
        baseRecyclerAdapter: BaseRecyclerAdapter<FaqInfo>,
        viewmodel: FaqListViewModel
    ): ViewHolder<FaqInfo>(itemBinding.root, baseRecyclerAdapter) {

        private val vm = viewmodel

        override fun populate(item: FaqInfo, position: Int) {
            itemBinding.data = item
            itemBinding.viewModel = vm

            val layoutExpand = itemBinding.linearFaqInfo
            itemBinding.layoutItem.setOnClickListener {
                val show = toggleLayout(!item.isExpanded, it, layoutExpand)
                item.isExpanded = show
            }
        }

        private fun toggleLayout(isExpanded: Boolean, view: View, layoutExpand: LinearLayout) : Boolean {
            ToggleAnimation.toggleArrow(view, isExpanded)
            if(isExpanded) {
                ToggleAnimation.expand(layoutExpand)
            } else {
                ToggleAnimation.collapse(layoutExpand)
            }
            return isExpanded
        }

    }

    class EmptyViewHolder(
        itemEmptyBinding: ItemEmptyBinding,
        baseRecyclerAdapter: BaseRecyclerAdapter<FaqInfo>
    ): ViewHolder<FaqInfo>(itemEmptyBinding.root, baseRecyclerAdapter)

}