package com.noorija.secret.guard.view.fragment

import android.app.Activity
import android.os.Bundle
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Toast
import androidx.databinding.DataBindingUtil
import androidx.databinding.ViewDataBinding
import androidx.fragment.app.Fragment
import com.noorija.secret.guard.viewmodel.BaseViewModel

abstract class BaseFragment<T : ViewDataBinding, R : BaseViewModel<*>> : Fragment() {
    private lateinit var currentView: View
    lateinit var viewDataBinding: T

    abstract val layoutResourceId: Int

    abstract val viewModel: R

    abstract fun initDataBinding()
    abstract fun initAfterBinding()

    private var isSetBackButtonValid = false

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?,
                              savedInstanceState: Bundle?): View? {
        viewDataBinding = DataBindingUtil.inflate(
            inflater, layoutResourceId, container, false)!!
        viewDataBinding.lifecycleOwner = this
        currentView = viewDataBinding.root
        return currentView
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        initDataBinding()
        initAfterBinding()
    }

    protected fun showToast(message: String?) {
        val toast: Toast = Toast.makeText(context, message, Toast.LENGTH_SHORT)
        toast.setGravity(Gravity.CENTER, 0, 0)
        toast.show()
    }

    protected fun showToast(message: Int) {
        val toast: Toast = Toast.makeText(context, message, Toast.LENGTH_SHORT)
        toast.setGravity(Gravity.CENTER, 0, 0)
        toast.show()
    }

    protected fun longToast(message: String?) {
        Toast.makeText(context, message, Toast.LENGTH_LONG).show()
    }

    protected fun longToast(message: Int) {
        Toast.makeText(context, message, Toast.LENGTH_LONG).show()
    }

    interface OnFragmentInteractionListener {
        fun onFragmentInteraction(code: Int, data: Any)
        //fun setDelivery()
    }

}