package com.noorija.secret.guard.view.fragment

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.animation.AnimationUtils
import android.widget.TextView
import androidx.fragment.app.viewModels
import androidx.core.content.ContextCompat
import com.google.android.material.tabs.TabLayout
import com.noorija.secret.guard.R
import com.noorija.secret.guard.databinding.FmtHomeBinding
import com.noorija.secret.guard.viewmodel.fragment.HomeFragmentViewModel
import com.noorija.secret.guard.view.adapter.HomeRecyclerAdapter
import com.noorija.secret.guard.view.adapter.TabsHomeAdapter

class HomeFragment : BaseFragment<FmtHomeBinding, HomeFragmentViewModel>() {
    companion object {
        @JvmStatic
        fun newInstance() =
            HomeFragment().apply {
                arguments = Bundle().apply {
                }
            }
    }

    override val layoutResourceId: Int
        get() = R.layout.fmt_home
    override val viewModel : HomeFragmentViewModel by viewModels()

    private lateinit var mRecyclerAdapter : HomeRecyclerAdapter

    override fun initDataBinding() {
        createTab()
    }

    override fun initAfterBinding() {
        viewDataBinding.viewModel = viewModel
    }

    private fun createTab() {
        val tabs = viewDataBinding.tabs
        tabs.addTab(tabs.newTab().setCustomView(createTabView(getString(R.string.tab_title_my_qrcode))))
        tabs.addTab(tabs.newTab().setCustomView(createTabView(getString(R.string.tab_title_call_history))))
        tabs.addTab(tabs.newTab().setCustomView(createTabView(getString(R.string.tab_title_public_reg))))

        val adapter = fragmentManager?.let { TabsHomeAdapter(it, tabs!!.tabCount) }
        viewDataBinding.tabsPager.adapter = adapter
        viewDataBinding.tabsPager.addOnPageChangeListener(TabLayout.TabLayoutOnPageChangeListener(tabs))
        tabs.addOnTabSelectedListener(object : TabLayout.OnTabSelectedListener {
            override fun onTabSelected(tab: TabLayout.Tab) {
                if(tab.position == 0) {
                    val slideDown = AnimationUtils.loadAnimation(requireContext(), R.anim.anim_slide_down)
                } else {
                    val slideUp = AnimationUtils.loadAnimation(requireContext(), R.anim.anim_slide_up)
                }
            }

            override fun onTabUnselected(tab: TabLayout.Tab?) {
            }

            override fun onTabReselected(tab: TabLayout.Tab?) {
            }
        })
        setTabSelected(0)
    }

    private fun createTabView(tabName: String) : View {
        val tabView: View = LayoutInflater.from(requireContext()).inflate(R.layout.tab_fmt_home, null)
        val tabTitle: TextView = tabView.findViewById(R.id.tvTabTitle)
        tabTitle.text = tabName
        return tabView
    }

    private fun setTabSelected(idx: Int) {
        for(i in 0 until viewDataBinding.tabs.tabCount) {
            val tabView = viewDataBinding.tabs.getTabAt(i)?.view
            val tabTitle: TextView = tabView?.findViewById(R.id.tvTabTitle)!!
            val tabInfo: TextView = tabView?.findViewById(R.id.tvTabInfo)!!
            if(i == idx) {
                tabTitle.setTextColor(ContextCompat.getColor(requireContext(), R.color.color_f9ac19))
                tabInfo.setTextColor(ContextCompat.getColor(requireContext(), R.color.color_f9ac19))
            } else {
                tabTitle.setTextColor(ContextCompat.getColor(requireContext(), R.color.color_3c3c3c))
                tabInfo.setTextColor(ContextCompat.getColor(requireContext(), R.color.color_3c3c3c))
            }
        }
    }

}