package com.noorija.secret.guard.view.fragment

import android.app.DownloadManager
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.database.Cursor
import android.net.Uri
import android.os.Bundle
import android.os.Environment
import android.widget.Toast
import androidx.fragment.app.viewModels
import androidx.lifecycle.Observer
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.noorija.secret.guard.R
import com.noorija.secret.guard.databinding.FmtMyQrcodeBinding
import com.noorija.secret.guard.rest.MyQrInfo
import com.noorija.secret.guard.text.PageAction
import com.noorija.secret.guard.view.InfoRegistActivity
import com.noorija.secret.guard.view.ScannerActivity
import com.noorija.secret.guard.view.adapter.MyQrcodeAdapter
import com.noorija.secret.guard.viewmodel.fragment.MyQrcodeFragmentViewModel
import timber.log.Timber
import java.io.File


class MyQrcodeFragment : BaseFragment<FmtMyQrcodeBinding, MyQrcodeFragmentViewModel>() {
    companion object {
        @JvmStatic
        fun newInstance() =
            MyQrcodeFragment().apply {
                arguments = Bundle().apply {
                }
            }
    }

    override val layoutResourceId: Int
        get() = R.layout.fmt_my_qrcode
    override val viewModel : MyQrcodeFragmentViewModel by viewModels()

    private lateinit var mRecyclerAdapter : MyQrcodeAdapter

    private var mDownloadManager: DownloadManager? = null
    private var mDownloadQueueId: Long? = null

    private val downloadCompleteReceiver: BroadcastReceiver = object : BroadcastReceiver() {
        override fun onReceive(context: Context, intent: Intent) {
            val reference = intent.getLongExtra(DownloadManager.EXTRA_DOWNLOAD_ID, -1)

            if (mDownloadQueueId === reference) {
                val query = DownloadManager.Query() // 다운로드 항목 조회에 필요한 정보 포함
                query.setFilterById(reference)
                val cursor: Cursor = mDownloadManager!!.query(query)

                cursor.moveToFirst()

                val columnIndex = cursor.getColumnIndex(DownloadManager.COLUMN_STATUS)
                val columnReason = cursor.getColumnIndex(DownloadManager.COLUMN_REASON)

                val status = cursor.getInt(columnIndex)
                val reason = cursor.getInt(columnReason)

                cursor.close()

                when (status) {
                    DownloadManager.STATUS_SUCCESSFUL -> Toast.makeText(
                        context,
                        context.getString(R.string.download_success),
                        Toast.LENGTH_SHORT
                    ).show()

                    DownloadManager.STATUS_PAUSED -> Toast.makeText(
                        context,
                        context.getString(R.string.download_pause),
                        Toast.LENGTH_SHORT
                    ).show()

                    DownloadManager.STATUS_FAILED -> Toast.makeText(
                        context,
                        context.getString(R.string.download_failed),
                        Toast.LENGTH_SHORT
                    ).show()
                }
            }
        }
    }

    override fun onResume() {
        viewModel.getMyQrList()
        super.onResume()
        val completeFilter = IntentFilter(DownloadManager.ACTION_DOWNLOAD_COMPLETE)
        context?.registerReceiver(downloadCompleteReceiver, completeFilter)
    }

    override fun initDataBinding() {
        viewDataBinding.viewModel = viewModel

        mRecyclerAdapter = MyQrcodeAdapter(viewModel)
        viewDataBinding.myQrRecyclerView.apply {
            adapter = mRecyclerAdapter
            layoutManager = LinearLayoutManager(requireContext(), RecyclerView.VERTICAL, false)
        }
    }

    private fun qrDown(data: MyQrInfo) {
        var qrNo = data.qrNo
        mDownloadManager = context?.getSystemService(Context.DOWNLOAD_SERVICE) as DownloadManager

        val file = File(File(Environment.getExternalStorageDirectory(), "Download"), "$qrNo.png")
        if (!file.parentFile.exists()) {
            file.parentFile.mkdirs()
        }

        val downloadUri: Uri = Uri.parse(data.filePath)
        val request = DownloadManager.Request(downloadUri)
        val pathSegmentList = downloadUri.pathSegments
        request.setTitle("다운로드 항목")
        request.setDestinationUri(Uri.fromFile(file))
        request.setAllowedOverMetered(true)

        mDownloadQueueId = mDownloadManager!!.enqueue(request)
    }

    override fun initAfterBinding() {
        with(viewModel) {
            scanEvent.observe(this@MyQrcodeFragment, Observer {
                startActivity(Intent(requireActivity(), ScannerActivity::class.java).apply {
                    putExtra("CallType", "RegQR")
                })
//                val intent = Intent(requireActivity(), ScannerActivity::class.java)
//                intent.addFlags(Intent.FLAG_ACTIVITY_NO_ANIMATION)
//                startActivity(intent)
            })
            downloadEvent.observe(this@MyQrcodeFragment, Observer {
                qrDown(it)
            })
            actionEvent.observe(this@MyQrcodeFragment, Observer {
                when (it.action) {
                    PageAction.DOWNLOAD -> {
                        qrDown(it.data)
                    }
                    PageAction.MODIFY -> {
                        val intent = Intent(requireActivity(), InfoRegistActivity::class.java)
                        intent.putExtra("MyQrInfo", it.data)
                        startActivity(intent)
                    }
                    PageAction.DELETE -> {
                        val title = "${it.data.nickNm}\n(${it.data.carInfo})"
                        val dialog = CustomDialogFragment(title, context.getString(R.string.msg_confirm_delete))
                        dialog.setButtonClickListener(object : CustomDialogFragment.OnButtonClickListener {
                            override fun onConfirmClicked() {
                                viewModel.deleteQrInfo(it.data)
                            }
                        })
                        dialog.show(requireActivity().supportFragmentManager, "MyQrInfo")
                    }
                }
            })
        }
    }

}