package com.noorija.secret.guard.viewmodel

import android.app.Application
import androidx.lifecycle.MutableLiveData
import com.noorija.secret.guard.rest.AskRequest
import com.noorija.secret.guard.rest.ExceptionDTO
import com.noorija.secret.guard.rest.SaveResponse
import com.noorija.secret.guard.text.PageView
import com.noorija.secret.guard.util.SingleLiveEvent
import com.noorija.secret.guard.util.SystemUtil

class AskListViewModel(application: Application): BaseViewModel<Any>(application) {

    val closeEvent: SingleLiveEvent<Any> = SingleLiveEvent()

    var isBtnSendEnable = MutableLiveData<Boolean>(false)
    val askEmail = MutableLiveData<String>("")
    val askSubject = MutableLiveData<String>("")
    val askDesc = MutableLiveData<String>("")

    fun onClickSend(){
        var askRequest = AskRequest(
            askEmail.value!!
            , askSubject.value!!
            , askDesc.value!!
            , SystemUtil.getSystemLanguage(context)
        )
        addDisposable(
            restClient.sendAsk(getToken(), askRequest)
            , dataObserver
        )
    }

    fun onTextChanged(mode: Int, s: CharSequence) {
        isBtnSendEnable.value = !(askEmail.value.isNullOrEmpty() || askSubject.value.isNullOrEmpty() || askDesc.value.isNullOrEmpty())
    }

    fun onClickClose(){
        closeEvent.value = PageView.BACK
    }

    override fun onRetrieveDataSuccess(data: Any) {
        when (data) {
            is SaveResponse -> {
                showToast(data.message)
                onClickClose()
            }
        }
    }

    override fun onRetrieveDataError(resError: ExceptionDTO?) {
        showToast(resError?.message)
    }

}