package com.noorija.secret.guard.viewmodel

import android.app.Application
import android.os.Bundle
import android.provider.Settings
import androidx.databinding.ObservableBoolean
import androidx.lifecycle.MutableLiveData
import com.noorija.secret.guard.rest.ExceptionDTO
import com.noorija.secret.guard.rest.MyQrInfo
import com.noorija.secret.guard.rest.QrMembRegRequest
import com.noorija.secret.guard.rest.QrRegResponse
import com.noorija.secret.guard.rest.SaveResponse
import com.noorija.secret.guard.text.PageView
import com.noorija.secret.guard.util.SingleLiveEvent
import com.noorija.secret.guard.util.SystemUtil

class InfoRegistViewModel(application: Application): BaseViewModel<Any>(application) {

    val infoRegistEvent: SingleLiveEvent<Any> = SingleLiveEvent()

    var strQrNo = ""
    val strAlarmName = MutableLiveData<String>("")
    val strCarType = MutableLiveData<String>("")
    val strCarNumber = MutableLiveData<String>("")
    val isPublicRegist = ObservableBoolean(false)
    val btnEnable = ObservableBoolean(false)

    private var isModify = false

    fun initViewModel(myQrInfo: MyQrInfo?) {
        if(myQrInfo != null) {
            isModify = true
            strQrNo = myQrInfo.qrNo.toString()
            strAlarmName.value = myQrInfo.nickNm
            strCarType.value = myQrInfo.makerCar
            strCarNumber.value = myQrInfo.carInfo
        }
    }

    fun onTextChanged(mode: Int, s: CharSequence) {
        btnEnable.set(s.toString().isNotEmpty())
    }

    fun onClickInfoReg() {
        val qrMembRegRequest = QrMembRegRequest(
            strQrNo
            , strAlarmName.value!!
            , strCarType.value!!
            , strCarNumber.value!!
            , Settings.Secure.getString(context.contentResolver, Settings.Secure.ANDROID_ID)
            , SystemUtil.getSystemLanguage(context)
        )
        if(isModify) {
            addDisposable(
                restClient.saveQrinfo(getToken(), qrMembRegRequest)
                , dataObserver
            )
        } else {
            addDisposable(
                restClient.regQr(getToken(), qrMembRegRequest)
                , dataObserver
            )
        }
    }

    override fun onRetrieveDataSuccess(data: Any) {
        when (data) {
            is QrRegResponse -> {
                infoRegistEvent.value = PageView.MAIN
            }
            is SaveResponse -> {
                showToast(data.message)
                infoRegistEvent.value = PageView.FINISH
            }
        }
    }

    override fun onRetrieveDataError(resError: ExceptionDTO?) {
        showToast(resError?.message)
    }

}