package com.noorija.secret.guard.viewmodel

import android.app.Application
import androidx.databinding.ObservableBoolean
import androidx.lifecycle.MutableLiveData
import com.noorija.secret.guard.rest.AppMenuListResponse
import com.noorija.secret.guard.rest.ExceptionDTO
import com.noorija.secret.guard.rest.RoomIdInfo
import com.noorija.secret.guard.text.FloatingMenuType
import com.noorija.secret.guard.util.SingleLiveEvent
import com.noorija.secret.guard.util.SystemUtil
import timber.log.Timber

class MainViewModel(application: Application): BaseViewModel<Any>(application) {

    val mainEvent: SingleLiveEvent<List<RoomIdInfo>> = SingleLiveEvent()
    val floatingButtonEvent: SingleLiveEvent<Boolean> = SingleLiveEvent()
    val floatingMenuEvent: SingleLiveEvent<FloatingMenuType> = SingleLiveEvent()

    var btnFloatingSetEnable = MutableLiveData<Boolean>(true)
    var btnFloatingFaqEnable = MutableLiveData<Boolean>(true)
    var btnFloatingAskEnable = MutableLiveData<Boolean>(true)
    var btnFloatingNoticeEnable = MutableLiveData<Boolean>(true)
    var btnFloatingManualEnable = MutableLiveData<Boolean>(true)
    var btnFloatingQrEnable = MutableLiveData<Boolean>(true)

    var isFloatingClose = MutableLiveData(false)
    var isFloatingVisibility = ObservableBoolean(false)

    init {
        menuList()
    }

    private fun menuList() {
        addDisposable(
            restClient.menuList(getToken(), SystemUtil.getSystemLanguage(context))
            , dataObserver
        )
    }

    fun onClickFloatingButton() {
        Timber.d("onClickFloatingButton =========> ")
        isFloatingClose.value = !isFloatingClose.value!!
        isFloatingVisibility.set(!isFloatingVisibility.get())
        floatingButtonEvent.value = isFloatingClose.value
    }

    fun onClickFloatingMenu(floatingMenuType: FloatingMenuType) {
        Timber.d("onClickFloatingMenu =========> %s", floatingMenuType)
//        var isBtnEnable = when(floatingMenuType) {
//            FloatingMenuType.SET -> btnFloatingSetEnable.value!!
//            FloatingMenuType.FAQ -> btnFloatingFaqEnable.value!!
//            FloatingMenuType.ASK -> btnFloatingAskEnable.value!!
//            FloatingMenuType.NOTICE -> btnFloatingNoticeEnable.value!!
//            FloatingMenuType.MANUAL -> btnFloatingManualEnable.value!!
//            FloatingMenuType.QR -> btnFloatingQrEnable.value!!
//            else -> false
//        }
//        if(isBtnEnable)
            floatingMenuEvent.value = floatingMenuType
    }

    override fun onRetrieveDataSuccess(data: Any) {
        when (data) {
            is AppMenuListResponse -> {
                data.listMenu.let { list ->
                    if (list != null) {
                        btnFloatingSetEnable.value = (list.filter { it.appMenuNo == "AP1004" })[0].useYn == "Y"
                        btnFloatingFaqEnable.value = (list.filter { it.appMenuNo == "AP1007" })[0].useYn == "Y"
                        btnFloatingAskEnable.value = (list.filter { it.appMenuNo == "AP1008" })[0].useYn == "Y"
                        btnFloatingNoticeEnable.value = (list.filter { it.appMenuNo == "AP1005" })[0].useYn == "Y"
                        btnFloatingManualEnable.value = (list.filter { it.appMenuNo == "AP1009" })[0].useYn == "Y"
                        btnFloatingQrEnable.value = (list.filter { it.appMenuNo == "AP1002" })[0].useYn == "Y"
                    }
                }
                mainEvent.value = data.roomIdList
            }
        }
    }

    override fun onRetrieveDataError(resError: ExceptionDTO?) {
        showToast(resError?.message)
    }

}