package com.noorija.secret.guard.viewmodel

import android.app.Application
import android.os.Build
import android.provider.Settings
import android.widget.RadioGroup
import androidx.lifecycle.MutableLiveData
import com.noorija.secret.guard.BuildConfig
import com.noorija.secret.guard.R
import com.noorija.secret.guard.model.CasAgreeInfo
import com.noorija.secret.guard.rest.ExceptionDTO
import com.noorija.secret.guard.rest.JwtResponse
import com.noorija.secret.guard.rest.MembRequest
import com.noorija.secret.guard.rest.SaveCasInfo
import com.noorija.secret.guard.text.Command
import com.noorija.secret.guard.text.Constants
import com.noorija.secret.guard.text.PageView
import com.noorija.secret.guard.util.PreferencesUtil
import com.noorija.secret.guard.util.SingleLiveEvent
import com.noorija.secret.guard.util.StringUtil
import com.noorija.secret.guard.util.SystemUtil
import com.noorija.secret.guard.util.ValidCheck
import timber.log.Timber

class MemberJoinViewModel(application: Application): BaseViewModel<Any>(application) {

    val memberJoinEvent: SingleLiveEvent<Any> = SingleLiveEvent()

    val strBirthYear = MutableLiveData<String>("")
    val strBirthMonth = MutableLiveData<String>("")
    val strBirthDay = MutableLiveData<String>("")
    val chkRbMan = MutableLiveData<Boolean>(true)
    val chkRbWoman = MutableLiveData<Boolean>(false)
    val strPhoneNo = MutableLiveData<String>("")
    private var strGender = "M"

    var btnEnable = MutableLiveData<Boolean>(false)
    private var isValidPhoneNo = MutableLiveData<Boolean>(false)

    private lateinit var casAgreeList : ArrayList<SaveCasInfo>

    init {
    }

    fun onTextChanged(mode: Int, s: CharSequence) {
        if(mode == 1) {
            var telNo = s.toString()
            telNo = telNo.replace("-","")
            isValidPhoneNo.value = ValidCheck.checkPhone(telNo)
        }
        chkJoinBtn()
    }

    fun onGenderChanged(radio: RadioGroup, id: Int) {
        strGender = if(id == R.id.rbWoman) "F" else "M"
    }

    private fun chkJoinBtn() {
        btnEnable.value = isValidPhoneNo.value
    }

    fun setCasAgreeList(casAgreeList : ArrayList<SaveCasInfo>) {
        this.casAgreeList = casAgreeList
    }

    fun onClickMemberReg() {
        val membRequest = MembRequest(
            Constants.REST_DEVICE_OS_TYPE_CODE
            , Settings.Secure.getString(context?.contentResolver, Settings.Secure.ANDROID_ID)
            , Build.VERSION.SDK_INT.toString()
            , BuildConfig.VERSION_NAME
            , strGender
            , strPhoneNo.value!!
            , strBirthYear.value + strBirthMonth.value + strBirthDay.value
            , SystemUtil.getSystemLanguage(context)
            , StringUtil.getCurrentDate()
            , this.casAgreeList
        )
        addDisposable(
            restClient.createSg(membRequest)
            , dataObserver
        )
    }

    override fun onRetrieveDataSuccess(data: Any) {
        when (data) {
            is JwtResponse -> {
                PreferencesUtil.putPref(Command.SECRET_GUARD_PREF_NAME_APP_KEY, data.appKey)
                memberJoinEvent.value = PageView.MAIN
            }
        }
    }

    override fun onRetrieveDataError(resError: ExceptionDTO?) {
        showToast(resError?.message)
    }

}