package com.noorija.secret.guard.viewmodel

import android.app.Application
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.noorija.secret.guard.model.Todo
import com.noorija.secret.guard.rest.AnnounceListRequest
import com.noorija.secret.guard.rest.CasMngInfo
import com.noorija.secret.guard.rest.ExceptionDTO
import com.noorija.secret.guard.rest.PublicInfo
import com.noorija.secret.guard.rest.PublicListResponse
import com.noorija.secret.guard.rest.SysVersionResponse
import com.noorija.secret.guard.text.Command
import com.noorija.secret.guard.text.PageView
import com.noorija.secret.guard.util.PreferencesUtil
import com.noorija.secret.guard.util.SingleLiveEvent
import com.noorija.secret.guard.util.StringUtil
import com.noorija.secret.guard.util.SystemUtil

class NoticeListViewModel(application: Application): BaseViewModel<Any>(application) {

    val closeEvent: SingleLiveEvent<Any> = SingleLiveEvent()

    private val recyclerList = mutableListOf<PublicInfo>()
    private var _noticeList = MutableLiveData<List<PublicInfo>>()
    val noticeList : LiveData<List<PublicInfo>>
        get() = _noticeList

    private val pageSize: Int = 10
    private var totalPage: Int = 1
    private var curPage: Int = 0

    init {
        reqNoticeList()
    }

    fun reqNoticeList() {
        if(totalPage <= curPage) {
            return
        }
        curPage++
        var announceListRequest = AnnounceListRequest(
            SystemUtil.getSystemLanguage(context)
            , StringUtil.getCurrentDate()
            , pageSize
            , curPage
            , "Y"
        )
        addDisposable(
            restClient.annList(getToken(), announceListRequest)
            , dataObserver
        )
    }

    fun onClickClose(){
        closeEvent.value = PageView.BACK
    }

    override fun onRetrieveDataSuccess(data: Any) {
        when (data) {
            is PublicListResponse -> {
                recyclerList.addAll(data.publicList)
                _noticeList.value = recyclerList
                totalPage = data.totalSize
            }
        }
    }

    override fun onRetrieveDataError(resError: ExceptionDTO?) {
        showToast(resError?.message)
    }

}