package com.noorija.secret.guard.viewmodel

import android.app.Application
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.noorija.secret.guard.model.Todo
import com.noorija.secret.guard.rest.ExceptionDTO
import com.noorija.secret.guard.rest.FaqListResponse
import com.noorija.secret.guard.rest.MembSettingRequest
import com.noorija.secret.guard.rest.MembSettingResponse
import com.noorija.secret.guard.text.PageView
import com.noorija.secret.guard.util.SingleLiveEvent
import com.noorija.secret.guard.util.StringUtil
import com.noorija.secret.guard.util.SystemUtil

class PreferencesViewModel(application: Application): BaseViewModel<Any>(application) {

    val closeEvent: SingleLiveEvent<Any> = SingleLiveEvent()
    val ringtoneEvent: SingleLiveEvent<Any> = SingleLiveEvent()

    var isCallAlarm = MutableLiveData<Boolean>(true)
    var isCallSound = MutableLiveData<Boolean>(true)
    var isVibration = MutableLiveData<Boolean>(true)
    var isCallMessage = MutableLiveData<Boolean>(true)
    var isVoiceTalk = MutableLiveData<Boolean>(true)
    var isAlarmMessage = MutableLiveData<Boolean>(true)

    private var strAlarmInterval = ""
    private var strAlarmSound = ""

    init {
        getInfoSetting()
    }

    private fun getInfoSetting() {
        addDisposable(
            restClient.infoSetting(getToken(), SystemUtil.getSystemLanguage(context))
            , dataObserver
        )
    }

    fun onClickSwitch(type: Int) {
        var isChecked: Boolean = false
        when(type) {
            10 -> {
                isChecked = isCallAlarm.value == true
                isCallAlarm.value = isChecked
            }
            20 -> {
                isChecked = isCallSound.value == true
                isCallSound.value = isChecked
            }
            30 -> {
                isChecked = isVibration.value == true
                isVibration.value = isChecked
            }
            40 -> {
                isChecked = isCallMessage.value == true
                isCallMessage.value = isChecked
            }
            50 -> {
                isChecked = isVoiceTalk.value == true
                isVoiceTalk.value = isChecked
            }
            60 -> {
                isChecked = isAlarmMessage.value == true
                isAlarmMessage.value = isChecked
            }
        }
    }

    fun setAlarmInterval(interval: String) {
        showToast(interval)
        strAlarmInterval = interval
    }

//    data class MembSettingRequest(var alarmUseYn: String, var alarmIntervalCd: String, var alarmSoundYn: String
//                                  , var alarmSoundNm: String, var alarmVibYn: String, var callPhoneYn: String
//                                  , var callMsgYn: String, var langCd: String, var currentDate: String = StringUtil.getCurrentDate())
    private fun saveSetting() {
        val strCallAlarm = if(isCallAlarm.value == true) "Y" else "N"
        val strCallSound = if(isCallSound.value == true) "Y" else "N"
        val strVibration = if(isVibration.value == true) "Y" else "N"
//        val strCallMessage = if(isCallMessage.value == true) "Y" else "N"
        val strVoiceTalk = if(isVoiceTalk.value == true) "Y" else "N"
        val strAlarmMessage = if(isAlarmMessage.value == true) "Y" else "N"
        val setting = MembSettingRequest(
            strCallAlarm, strAlarmInterval, strCallSound, strAlarmSound, strVibration
            , strVoiceTalk, strAlarmMessage, SystemUtil.getSystemLanguage(context)
        )
        addDisposable(
            restClient.saveSetting(getToken(), setting)
            , dataObserver
        )
    }

    fun onClickSetRingtone() {
        ringtoneEvent.value = "Ringtone"
    }

    fun onClickClose(){
        closeEvent.value = PageView.BACK
    }

    override fun onRetrieveDataSuccess(data: Any) {
        when (data) {
            is MembSettingResponse -> {
                isCallAlarm.value = data.alarmUseYn == "Y"
                isCallSound.value = data.alarmSoundYn == "Y"
                isVibration.value = data.alarmVibYn == "Y"
                isCallMessage.value = data.allowYn == "Y"
                isVoiceTalk.value = data.callPhoneYn == "Y"
                isAlarmMessage.value = data.callMsgYn == "Y"
                strAlarmInterval = data.alarmIntervalCd
            }
        }
    }

    override fun onRetrieveDataError(resError: ExceptionDTO?) {
        showToast(resError?.message)
    }

}