package com.noorija.secret.guard.viewmodel

import android.app.Application
import androidx.databinding.ObservableBoolean
import androidx.databinding.ObservableField
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.google.gson.GsonBuilder
import com.noorija.secret.guard.R
import com.noorija.secret.guard.model.StompPayload
import com.noorija.secret.guard.rest.ExceptionDTO
import com.noorija.secret.guard.rest.FaqInfo
import com.noorija.secret.guard.rest.FaqListResponse
import com.noorija.secret.guard.text.PageView
import com.noorija.secret.guard.util.SingleLiveEvent
import com.noorija.secret.guard.util.SystemUtil
import com.noorija.secret.guard.util.ValidCheck
import timber.log.Timber

class RcvMsgViewModel(application: Application): BaseViewModel<Any>(application) {

    val closeEvent: SingleLiveEvent<Any> = SingleLiveEvent()
    val sendMessageEvent: SingleLiveEvent<Any> = SingleLiveEvent()
    val sendCallEvent: SingleLiveEvent<Any> = SingleLiveEvent()

    val strNickNm = MutableLiveData<String>("")
    val strRequestMessage = MutableLiveData<String>("")
    val strInputSendMessage = MutableLiveData<String>("")
    var strMoveImmediate = ObservableField(getApplication<Application>().resources.getString(R.string.msg_move_immediate))
    var strSendMessage = ObservableField(getApplication<Application>().resources.getString(R.string.msg_send_message))

    var isRcvMsg = ObservableBoolean(true)

    fun init(nickNm: String, strPayload: String) {
        val gson = GsonBuilder()
            .disableHtmlEscaping()
            .serializeNulls()
            .create()
        val payload = gson.fromJson(strPayload, StompPayload::class.java)

        if(payload.mType == "M") {
            isRcvMsg.set(true)
            strNickNm.value = nickNm + " " + getApplication<Application>().resources.getString(R.string.lbl_phone_message)
            strRequestMessage.value = payload.msgDesc
        } else if(payload.mType == "C") {
            isRcvMsg.set(false)
            strNickNm.value = nickNm + " " + getApplication<Application>().resources.getString(R.string.lbl_phone_call)
        }
    }

    fun onTextChanged(mode: Int, s: CharSequence) {
        strInputSendMessage.value = s.toString()
    }

    fun onClickSendMessage(idx: Int){
        Timber.d("onClickSendMessage >>>>>>>>>>>>>>>>> $idx")
        if(idx == 2 && strInputSendMessage.value.isNullOrEmpty())
            return
        var sendMessage = ""
        when(idx) {
            0 -> sendMessage = strMoveImmediate.get().toString()
            1 -> sendMessage = strSendMessage.get().toString()
            2 -> sendMessage = strInputSendMessage.value.toString()
        }
        Timber.d("onClickSendMessage sendMessage >>>>>>>>>>>>>>>>> $sendMessage")
        sendMessageEvent.value = sendMessage
    }

    fun onClickSendCall(idx: Int){
        Timber.d("onClickSendCall >>>>>>>>>>>>>>>>> $idx")
        when(idx) {
            0 -> sendCallEvent.value = ""
        }
    }

    fun onClickClose(){
        closeEvent.value = PageView.FINISH
    }

    override fun onRetrieveDataSuccess(data: Any) {
        when (data) {
        }
    }

    override fun onRetrieveDataError(resError: ExceptionDTO?) {
        showToast(resError?.message)
    }

}