package com.noorija.secret.guard.viewmodel

import android.Manifest
import android.annotation.SuppressLint
import android.app.Application
import android.content.Intent
import android.net.UrlQuerySanitizer
import androidx.activity.result.ActivityResultLauncher
import androidx.activity.result.contract.ActivityResultContracts
import com.google.zxing.integration.android.IntentIntegrator
import com.google.zxing.integration.android.IntentResult
import com.noorija.secret.guard.R
import com.noorija.secret.guard.rest.ExceptionDTO
import com.noorija.secret.guard.rest.QRExistResponse
import com.noorija.secret.guard.rest.QrMembRegRequest
import com.noorija.secret.guard.rest.QrUseRegRequest
import com.noorija.secret.guard.rest.SaveResponse
import com.noorija.secret.guard.text.Command
import com.noorija.secret.guard.text.PageView
import com.noorija.secret.guard.util.PreferencesUtil
import com.noorija.secret.guard.util.SingleLiveEvent
import com.noorija.secret.guard.util.SystemUtil
import com.noorija.secret.guard.view.CustomScannerActivity
import com.noorija.secret.guard.view.ScannerActivity
import timber.log.Timber
import java.net.URLEncoder

class ScannerViewModel(application: Application): BaseViewModel<Any>(application) {
    companion object {
        private const val REQUEST_CODE_BARCODE = 200
    }

    val scanEvent: SingleLiveEvent<String> = SingleLiveEvent()
    val flashEvent: SingleLiveEvent<Boolean> = SingleLiveEvent()
    val closeEvent: SingleLiveEvent<Any> = SingleLiveEvent()

    private var isFlashOn = false
    private var qrNo = ""

    lateinit var act: ScannerActivity

    private var resultLauncher: ActivityResultLauncher<Intent>? = null

    init {
    }

    fun initViewModel(act: ScannerActivity) {
        this.act = act
        setResultLauncher()
        if(isPermission(Manifest.permission.CAMERA)) {
            startScanner()
        } else {
            requestPermissions(Manifest.permission.CAMERA) {
                if(it) startScanner() else closeEvent.value = PageView.BACK
            }
        }
    }

    private fun setResultLauncher() {
        resultLauncher = act.registerForActivityResult(ActivityResultContracts.StartActivityForResult()) { result ->
            val intentResult: IntentResult? = IntentIntegrator.parseActivityResult(result.resultCode, result.data)
            if (intentResult != null) {
                Timber.d("Scan Data >>>>>>>>>>>>>>>> %s", intentResult.contents)
                if(intentResult.contents.isNullOrEmpty()) {
                    showToast(context.getString(R.string.toast_scan_empty))
                    scanEvent.value = qrNo
                } else {
                    if(act.callType == "WebRTC") {
                        scanEvent.value = intentResult.contents
                    } else {
                        val sanitizer = UrlQuerySanitizer(intentResult.contents)
                        qrNo = sanitizer.getValue("qrNo")
                        regUseQr()
                    }
                }
            } else {
                showToast(context.getString(R.string.toast_scan_failed))
                scanEvent.value = qrNo
            }
        }
    }

    private fun startScanner() {
        val intentIntegrator = IntentIntegrator(act)
        intentIntegrator.setBeepEnabled(false) //바코드 인식시 소리
        intentIntegrator.setOrientationLocked(false)
        intentIntegrator.setRequestCode(REQUEST_CODE_BARCODE)
        intentIntegrator.captureActivity = CustomScannerActivity::class.java
//        act?.runOnUiThread(Runnable { intentIntegrator.initiateScan() })
        act.runOnUiThread(Runnable { resultLauncher?.launch(intentIntegrator.createScanIntent()) })
    }

    fun onClickClose() {
        closeEvent.value = PageView.BACK
    }

    private fun regUseQr() {
        val qrUseRegRequest = QrUseRegRequest(qrNo, SystemUtil.getSystemLanguage(context))
        addDisposable(
            restClient.regUseQr(getToken(), qrUseRegRequest)
            , dataObserver
        )
//        addDisposable(
//            restClient.regUseQr(getToken(), qrNo, SystemUtil.getSystemLanguage(context))
//            , dataObserver
//        )
    }

    override fun onRetrieveDataSuccess(data: Any) {
        when (data) {
            is QRExistResponse -> {
                when (data.regQr) {
                    "N" -> scanEvent.value = qrNo
                    "D" -> scanEvent.value = qrNo
                    else -> {
                        showToast(data.message)
                        onClickClose()
                    }
                }
            }
        }
    }

    override fun onRetrieveDataError(resError: ExceptionDTO?) {
        showToast(resError?.message)
    }

}