package com.noorija.secret.guard.viewmodel

import android.annotation.SuppressLint
import android.app.Application
import android.os.Build
import android.provider.Settings
import com.noorija.secret.guard.BuildConfig
import com.noorija.secret.guard.rest.ExceptionDTO
import com.noorija.secret.guard.rest.SysVersionResponse
import com.noorija.secret.guard.rest.VerInfoRequest
import com.noorija.secret.guard.text.AppUpgradeType
import com.noorija.secret.guard.text.Command
import com.noorija.secret.guard.text.Constants
import com.noorija.secret.guard.text.PageView
import com.noorija.secret.guard.util.PreferencesUtil
import com.noorija.secret.guard.util.SingleLiveEvent
import com.noorija.secret.guard.util.SystemUtil

@SuppressLint("HardwareIds")
class SplashViewModel(application: Application): BaseViewModel<Any>(application) {

    val upgradeEvent: SingleLiveEvent<AppUpgradeType> = SingleLiveEvent()
    val finishEvent: SingleLiveEvent<PageView> = SingleLiveEvent()

    lateinit var sysVersion: SysVersionResponse

    init {
        getAppVer()
    }

    private fun getAppVer() {
        val verInfoRequest = VerInfoRequest(
            Constants.REST_DEVICE_OS_TYPE_CODE, Settings.Secure.getString(context.contentResolver, Settings.Secure.ANDROID_ID)
            , Build.VERSION.SDK_INT.toString(), BuildConfig.VERSION_NAME, SystemUtil.getSystemLanguage(context)
        )
        addDisposable(
            restClient.getVersion(verInfoRequest)
            , dataObserver
        )
    }

    override fun onRetrieveDataSuccess(data: Any) {
        when (data) {
            is SysVersionResponse -> {
                if(data.appKey.isNotEmpty())
                    PreferencesUtil.putPref(Command.SECRET_GUARD_PREF_NAME_APP_KEY, data.appKey)
                if(data.upRequireInfo == "10") {
                    if(data.tutoStart == "Y") {
                        finishEvent.value = PageView.TUTORIAL
                    } else {
                        finishEvent.value = PageView.MAIN
                    }
                } else {
                    this.sysVersion = data
                    finishEvent.value = PageView.DIALOG
                }
            }
        }
    }

    override fun onRetrieveDataError(resError: ExceptionDTO?) {
        showToast(resError?.message)
        finishEvent.value = PageView.FINISH
    }

}