package com.noorija.secret.guard.viewmodel

import android.Manifest
import android.app.Application
import android.view.View
import androidx.databinding.ObservableBoolean
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.noorija.secret.guard.model.CasAgreeInfo
import com.noorija.secret.guard.rest.CasMngInfo
import com.noorija.secret.guard.rest.CasMngListResponse
import com.noorija.secret.guard.rest.ExceptionDTO
import com.noorija.secret.guard.rest.SaveCasInfo
import com.noorija.secret.guard.rest.TutoInfo
import com.noorija.secret.guard.rest.TutoListResponse
import com.noorija.secret.guard.util.SingleLiveEvent
import com.noorija.secret.guard.util.SystemUtil
import timber.log.Timber


class TutorialViewModel(application: Application): BaseViewModel<Any>(application) {
    private val permissions = arrayOf(
        Manifest.permission.READ_PHONE_STATE,
        Manifest.permission.CAMERA,
        Manifest.permission.READ_EXTERNAL_STORAGE
    )

    val tutorialEvent: SingleLiveEvent<Any> = SingleLiveEvent()
    val joinEvent: SingleLiveEvent<ArrayList<SaveCasInfo>> = SingleLiveEvent()

    private var requiredCasCount = 0
    private var _casList = MutableLiveData<List<CasMngInfo>>()
    val casList : LiveData<List<CasMngInfo>>
        get() = _casList

    var isCheckAll = ObservableBoolean(false)
    var bottomVisibility = ObservableBoolean(false)
    val btnEnable = ObservableBoolean(false)

    init {
        getTutorial()
    }

    private fun getTutorial() {
        addDisposable(
            restClient.getTutorial(SystemUtil.getSystemLanguage(context))
            , dataObserver
        )
    }

    fun getCas() {
        if(!_casList.value.isNullOrEmpty()) return

        addDisposable(
            restClient.getCas(SystemUtil.getSystemLanguage(context))
            , dataObserver
        )
    }

    fun onClickAgreeAll(view: View) {
        casList.value.let {
            it?.forEach { casData ->
                casData.checked.set(isCheckAll.get())
            }
            btnEnable.set(isCheckAll.get())
        }
    }

    fun onClickCasAgree(view: View) {
        casList.value.let { list ->
            if (list != null) {
                isCheckAll.set(list.filter { it.checked.get() }.size == list.size)
                btnEnable.set(list.filter {
                    it.checked.get() && it.requiredYn == "Y"
                }.size == requiredCasCount)
            }
        }
    }

    fun onClickCasView(item: CasMngInfo) {
        tutorialEvent.value = item
    }

    fun onClickStart() {
        val casAgreeList = ArrayList<SaveCasInfo>()
        casList.value.let {
            it?.forEach { casData ->
                casAgreeList.add(SaveCasInfo(casData.casNo, if(casData.checked.get()) "Y" else "N"))
            }
        }
        joinEvent.value = casAgreeList
    }

    override fun onRetrieveDataSuccess(data: Any) {
        when (data) {
            is TutoListResponse -> {
                requestPermissions(permissions) { }
                val arrTutorialImage = ArrayList<String>()
                for(item: TutoInfo in data.tutoDataList) {
                    arrTutorialImage.add(item.filePath)
                }
                tutorialEvent.value = arrTutorialImage
            }
            is CasMngListResponse -> {
                _casList.value = data.casMngDataList
                requiredCasCount = casList.value?.filter { it.requiredYn == "Y" }?.size!!
            }
        }
    }

    override fun onRetrieveDataError(resError: ExceptionDTO?) {
        showToast(resError?.message)
    }

}