package com.noorija.secret.guard.viewmodel.fragment

import android.app.Application
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.noorija.secret.guard.rest.CallHistoryRequest
import com.noorija.secret.guard.rest.ExceptionDTO
import com.noorija.secret.guard.rest.CallHistory
import com.noorija.secret.guard.rest.CallHistoryResponse
import com.noorija.secret.guard.util.SystemUtil
import com.noorija.secret.guard.viewmodel.BaseViewModel

class CallHistoryFragmentViewModel(application: Application) : BaseViewModel<Any>(application) {

    private val recyclerList = mutableListOf<CallHistory>()
    private var _dataList = MutableLiveData<List<CallHistory>>()
    val dataList : LiveData<List<CallHistory>>
        get() = _dataList

    private val pageSize: Int = 10
    private var totalPage: Int = 1
    private var curPage: Int = 0

    init {
        getCallHistoryList()
    }

    fun getCallHistoryList() {
        if(totalPage <= curPage) {
            return
        }
        curPage++
        var callHistoryRequest = CallHistoryRequest(
            pageSize
            , curPage
            , "Y"
            , SystemUtil.getSystemLanguage(context)
        )
        addDisposable(
            restClient.callHist(getToken(), callHistoryRequest)
            , dataObserver
        )
    }

    override fun onRetrieveDataSuccess(data: Any) {
        when (data) {
            is CallHistoryResponse -> {
                recyclerList.addAll(data.callHistList)
                _dataList.value = recyclerList
                totalPage = data.totalSize
            }
        }
    }

    override fun onRetrieveDataError(resError: ExceptionDTO?) {
        showToast(resError?.message)
    }

}