package com.noorija.secret.guard.viewmodel.fragment

import android.app.Application
import android.os.Environment
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.noorija.secret.guard.R
import com.noorija.secret.guard.model.QrItemAct
import com.noorija.secret.guard.rest.CasMngInfo
import com.noorija.secret.guard.rest.ExceptionDTO
import com.noorija.secret.guard.rest.MyQrInfo
import com.noorija.secret.guard.rest.MyQrListResponse
import com.noorija.secret.guard.rest.QrMembDelRequest
import com.noorija.secret.guard.rest.SaveResponse
import com.noorija.secret.guard.text.Command
import com.noorija.secret.guard.text.DialogResultType
import com.noorija.secret.guard.text.DialogType
import com.noorija.secret.guard.text.PageAction
import com.noorija.secret.guard.text.PageView
import com.noorija.secret.guard.util.PreferencesUtil
import com.noorija.secret.guard.util.SingleLiveEvent
import com.noorija.secret.guard.util.SystemUtil
import com.noorija.secret.guard.view.MainActivity
import com.noorija.secret.guard.view.TutorialActivity
import com.noorija.secret.guard.view.fragment.CustomDialogFragment
import com.noorija.secret.guard.viewmodel.BaseViewModel
import timber.log.Timber
import java.io.File

class MyQrcodeFragmentViewModel(application: Application) : BaseViewModel<Any>(application) {

    val scanEvent: SingleLiveEvent<PageView> = SingleLiveEvent()
    val downloadEvent: SingleLiveEvent<MyQrInfo> = SingleLiveEvent()
    val actionEvent: SingleLiveEvent<QrItemAct> = SingleLiveEvent()

    private var _dataList = MutableLiveData<List<MyQrInfo>>()
    val dataList : LiveData<List<MyQrInfo>>
        get() = _dataList

    init {
        getMyQrList()
    }

    fun getMyQrList() {
        addDisposable(
            restClient.myQrList(getToken(), SystemUtil.getSystemLanguage(context))
            , dataObserver
        )
    }

    fun onClickDownload(data: MyQrInfo) {
        actionEvent.value = QrItemAct(PageAction.DOWNLOAD, data)
    }

    fun onClickChange(data: MyQrInfo) {
        actionEvent.value = QrItemAct(PageAction.MODIFY, data)
    }

    fun onClickDelete(data: MyQrInfo) {
        actionEvent.value = QrItemAct(PageAction.DELETE, data)
    }

    fun onClickScanner() {
        scanEvent.value = PageView.SCANNER
    }

    fun deleteQrInfo(data: MyQrInfo) {
        addDisposable(
            restClient.deleteQrinfo(getToken(), QrMembDelRequest(data.qrNo, SystemUtil.getSystemLanguage(context)))
            , dataObserver
        )
    }

    override fun onRetrieveDataSuccess(data: Any) {
        when (data) {
            is MyQrListResponse -> {
                _dataList.value = data.myQrList
            }
            is SaveResponse -> {
                showToast(data.message)
                getMyQrList()
            }
        }
    }

    override fun onRetrieveDataError(resError: ExceptionDTO?) {
        showToast(resError?.message)
    }

    override fun onDialogResult(id: Int, result: DialogResultType) {
        when (result) {
            DialogResultType.CANCEL -> { }
            else -> {
            }
        }
    }

    override fun onDialogResult(id: Int, data: Any, result: DialogResultType) {
        val qrInfo = data as MyQrInfo
        addDisposable(
            restClient.deleteQrinfo(getToken(), QrMembDelRequest(qrInfo.qrNo, SystemUtil.getSystemLanguage(context)))
            , dataObserver
        )
    }

}